// Generated by gen-type-expect-tests.py

package frysk.debuginfo;

import java.io.*;
import lib.dwfl.DwarfDie;
import lib.dwfl.Dwfl;
import lib.dwfl.DwflDieBias;
import frysk.dwfl.DwflCache;
import frysk.proc.Task;
import frysk.testbed.DaemonBlockedAtSignal;
import frysk.testbed.TestLib;
import frysk.value.Type;
import frysk.value.Value;
import frysk.value.Format;
import frysk.debuginfo.DebugInfo;


public class TestTypeFactory extends TestLib {
    private class TypeTestbed {
	DebugInfoFrame frame;
        Task task;
	DwarfDie die;
	DwarfDie[] allDies;
	TypeFactory typeFactory;
	String testName;

	TypeTestbed(String executable, String testName) {
	    task = new DaemonBlockedAtSignal(executable).getMainTask();
            frame = DebugInfoStackFactory.createDebugInfoStackTrace(task);
	    long pc = frame.getAdjustedAddress();
	    Dwfl dwfl = DwflCache.getDwfl(frame.getTask());
	    DwflDieBias bias = dwfl.getCompilationUnit(pc);
	    die = bias.die;
	    allDies = die.getScopes(pc - bias.bias);
	    typeFactory = new TypeFactory(frame.getTask().getISA());
	    this.testName = testName;
	}


	void checkType(String symbol, String expected) {
	    Type varType;

	    DwarfDie varDie = die.getScopeVar(allDies, symbol);
	    if (varDie == null)
		System.out.println("Error: Cannot find " + symbol);
	    assertNotNull(varDie);
	    varType = typeFactory.getType(varDie.getType());
	    assertNotNull(varType);
	    // System.out.println("Expect: " + symbol + "\n'" +
	    //    expected + "'\nGot:\n'" + varType.toPrint());
	    assertEquals(testName + symbol, expected, varType.toPrint());
	}

        void checkValue(String symbol, String expected) {
            // "Print" to a byte array
	    ByteArrayOutputStream baos = new ByteArrayOutputStream(32);
	    PrintWriter pw = new PrintWriter(baos, true);
	    // ??? cache address of x so &x can be checked?
	    if (expected.indexOf("&") >= 0 || symbol.indexOf("ptr") >= 0
		|| expected.length() == 0)
		return;
	    DwarfDie varDie = die.getScopeVar(allDies, symbol);
	    if (varDie == null)
		System.out.println("Error: Cannot find " + symbol);
	    assertNotNull(varDie);
	    DebugInfo debugInfo = new DebugInfo(frame);
	    Value value =  debugInfo.print(symbol, frame);
	    value.toPrint(pw, task.getMemory(), Format.NATURAL, 0);
	    pw.flush();
	    String valueString = baos.toString();
	    // System.out.println("Expect: " + symbol +
	    //     "\n'" + expected + "'\nGot:\n'" +
	    //     valueString + "'" + " " + value.getType());
	    assertEquals(testName + symbol, expected, valueString);
	    baos.reset();
	}
    }

    public void test_char_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testchar_var");
	typeTestbed.checkType("char_var","char");
	typeTestbed.checkValue("char_var","'~'");
    }
    public void test_const_char_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testconst_char_var");
	typeTestbed.checkType("const_char_var","const char");
	typeTestbed.checkValue("const_char_var","'!'");
    }
    public void test_volatile_char_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testvolatile_char_var");
	typeTestbed.checkType("volatile_char_var","volatile char");
	typeTestbed.checkValue("volatile_char_var","'!'");
    }
    public void test_ptr_char_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_char_var");
	typeTestbed.checkType("ptr_char_var","char *");
	typeTestbed.checkValue("ptr_char_var","&char_var");
    }
    public void test_ptr_const_char_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_const_char_var");
	typeTestbed.checkType("ptr_const_char_var","const char *");
	typeTestbed.checkValue("ptr_const_char_var","&char_var");
    }
    public void test_char_ptr_const_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testchar_ptr_const_var");
	typeTestbed.checkType("char_ptr_const_var","char * const");
	typeTestbed.checkValue("char_ptr_const_var","&char_var");
    }
    public void test_ptr_volatile_char_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_volatile_char_var");
	typeTestbed.checkType("ptr_volatile_char_var","volatile char *");
	typeTestbed.checkValue("ptr_volatile_char_var","&char_var");
    }
    public void test_char_ptr_volatile_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testchar_ptr_volatile_var");
	typeTestbed.checkType("char_ptr_volatile_var","char * volatile");
	typeTestbed.checkValue("char_ptr_volatile_var","&char_var");
    }
    public void test_short_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testshort_int_var");
	typeTestbed.checkType("short_int_var","short int");
	typeTestbed.checkValue("short_int_var","32767");
    }
    public void test_const_short_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testconst_short_int_var");
	typeTestbed.checkType("const_short_int_var","const short int");
	typeTestbed.checkValue("const_short_int_var","-32767");
    }
    public void test_volatile_short_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testvolatile_short_int_var");
	typeTestbed.checkType("volatile_short_int_var","volatile short int");
	typeTestbed.checkValue("volatile_short_int_var","-32767");
    }
    public void test_ptr_short_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_short_int_var");
	typeTestbed.checkType("ptr_short_int_var","short int *");
	typeTestbed.checkValue("ptr_short_int_var","&short_int_var");
    }
    public void test_ptr_const_short_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_const_short_int_var");
	typeTestbed.checkType("ptr_const_short_int_var","const short int *");
	typeTestbed.checkValue("ptr_const_short_int_var","&short_int_var");
    }
    public void test_short_int_ptr_const_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testshort_int_ptr_const_var");
	typeTestbed.checkType("short_int_ptr_const_var","short int * const");
	typeTestbed.checkValue("short_int_ptr_const_var","&short_int_var");
    }
    public void test_ptr_volatile_short_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_volatile_short_int_var");
	typeTestbed.checkType("ptr_volatile_short_int_var","volatile short int *");
	typeTestbed.checkValue("ptr_volatile_short_int_var","&short_int_var");
    }
    public void test_short_int_ptr_volatile_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testshort_int_ptr_volatile_var");
	typeTestbed.checkType("short_int_ptr_volatile_var","short int * volatile");
	typeTestbed.checkValue("short_int_ptr_volatile_var","&short_int_var");
    }
    public void test_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testint_var");
	typeTestbed.checkType("int_var","int");
	typeTestbed.checkValue("int_var","65536");
    }
    public void test_const_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testconst_int_var");
	typeTestbed.checkType("const_int_var","const int");
	typeTestbed.checkValue("const_int_var","-65536");
    }
    public void test_volatile_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testvolatile_int_var");
	typeTestbed.checkType("volatile_int_var","volatile int");
	typeTestbed.checkValue("volatile_int_var","-65536");
    }
    public void test_ptr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_int_var");
	typeTestbed.checkType("ptr_int_var","int *");
	typeTestbed.checkValue("ptr_int_var","&int_var");
    }
    public void test_ptr_const_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_const_int_var");
	typeTestbed.checkType("ptr_const_int_var","const int *");
	typeTestbed.checkValue("ptr_const_int_var","&int_var");
    }
    public void test_int_ptr_const_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testint_ptr_const_var");
	typeTestbed.checkType("int_ptr_const_var","int * const");
	typeTestbed.checkValue("int_ptr_const_var","&int_var");
    }
    public void test_ptr_volatile_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_volatile_int_var");
	typeTestbed.checkType("ptr_volatile_int_var","volatile int *");
	typeTestbed.checkValue("ptr_volatile_int_var","&int_var");
    }
    public void test_int_ptr_volatile_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testint_ptr_volatile_var");
	typeTestbed.checkType("int_ptr_volatile_var","int * volatile");
	typeTestbed.checkValue("int_ptr_volatile_var","&int_var");
    }
    public void test_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testlong_int_var");
	typeTestbed.checkType("long_int_var","long int");
	typeTestbed.checkValue("long_int_var","65536");
    }
    public void test_const_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testconst_long_int_var");
	typeTestbed.checkType("const_long_int_var","const long int");
	typeTestbed.checkValue("const_long_int_var","-65536");
    }
    public void test_volatile_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testvolatile_long_int_var");
	typeTestbed.checkType("volatile_long_int_var","volatile long int");
	typeTestbed.checkValue("volatile_long_int_var","-65536");
    }
    public void test_ptr_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_long_int_var");
	typeTestbed.checkType("ptr_long_int_var","long int *");
	typeTestbed.checkValue("ptr_long_int_var","&long_int_var");
    }
    public void test_ptr_const_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_const_long_int_var");
	typeTestbed.checkType("ptr_const_long_int_var","const long int *");
	typeTestbed.checkValue("ptr_const_long_int_var","&long_int_var");
    }
    public void test_long_int_ptr_const_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testlong_int_ptr_const_var");
	typeTestbed.checkType("long_int_ptr_const_var","long int * const");
	typeTestbed.checkValue("long_int_ptr_const_var","&long_int_var");
    }
    public void test_ptr_volatile_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_volatile_long_int_var");
	typeTestbed.checkType("ptr_volatile_long_int_var","volatile long int *");
	typeTestbed.checkValue("ptr_volatile_long_int_var","&long_int_var");
    }
    public void test_long_int_ptr_volatile_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testlong_int_ptr_volatile_var");
	typeTestbed.checkType("long_int_ptr_volatile_var","long int * volatile");
	typeTestbed.checkValue("long_int_ptr_volatile_var","&long_int_var");
    }
    public void test_long_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testlong_long_int_var");
	typeTestbed.checkType("long_long_int_var","long long int");
	typeTestbed.checkValue("long_long_int_var","65536");
    }
    public void test_const_long_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testconst_long_long_int_var");
	typeTestbed.checkType("const_long_long_int_var","const long long int");
	typeTestbed.checkValue("const_long_long_int_var","-65536");
    }
    public void test_volatile_long_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testvolatile_long_long_int_var");
	typeTestbed.checkType("volatile_long_long_int_var","volatile long long int");
	typeTestbed.checkValue("volatile_long_long_int_var","-65536");
    }
    public void test_ptr_long_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_long_long_int_var");
	typeTestbed.checkType("ptr_long_long_int_var","long long int *");
	typeTestbed.checkValue("ptr_long_long_int_var","&long_long_int_var");
    }
    public void test_ptr_const_long_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_const_long_long_int_var");
	typeTestbed.checkType("ptr_const_long_long_int_var","const long long int *");
	typeTestbed.checkValue("ptr_const_long_long_int_var","&long_long_int_var");
    }
    public void test_long_long_int_ptr_const_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testlong_long_int_ptr_const_var");
	typeTestbed.checkType("long_long_int_ptr_const_var","long long int * const");
	typeTestbed.checkValue("long_long_int_ptr_const_var","&long_long_int_var");
    }
    public void test_ptr_volatile_long_long_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_volatile_long_long_int_var");
	typeTestbed.checkType("ptr_volatile_long_long_int_var","volatile long long int *");
	typeTestbed.checkValue("ptr_volatile_long_long_int_var","&long_long_int_var");
    }
    public void test_long_long_int_ptr_volatile_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testlong_long_int_ptr_volatile_var");
	typeTestbed.checkType("long_long_int_ptr_volatile_var","long long int * volatile");
	typeTestbed.checkValue("long_long_int_ptr_volatile_var","&long_long_int_var");
    }
    public void test_float_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testfloat_var");
	typeTestbed.checkType("float_var","float");
	typeTestbed.checkValue("float_var","3.402823E38");
    }
    public void test_const_float_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testconst_float_var");
	typeTestbed.checkType("const_float_var","const float");
	typeTestbed.checkValue("const_float_var","1.1754939E-38");
    }
    public void test_volatile_float_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testvolatile_float_var");
	typeTestbed.checkType("volatile_float_var","volatile float");
	typeTestbed.checkValue("volatile_float_var","1.1754939E-38");
    }
    public void test_ptr_float_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_float_var");
	typeTestbed.checkType("ptr_float_var","float *");
	typeTestbed.checkValue("ptr_float_var","&float_var");
    }
    public void test_ptr_const_float_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_const_float_var");
	typeTestbed.checkType("ptr_const_float_var","const float *");
	typeTestbed.checkValue("ptr_const_float_var","&float_var");
    }
    public void test_float_ptr_const_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testfloat_ptr_const_var");
	typeTestbed.checkType("float_ptr_const_var","float * const");
	typeTestbed.checkValue("float_ptr_const_var","&float_var");
    }
    public void test_ptr_volatile_float_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_volatile_float_var");
	typeTestbed.checkType("ptr_volatile_float_var","volatile float *");
	typeTestbed.checkValue("ptr_volatile_float_var","&float_var");
    }
    public void test_float_ptr_volatile_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testfloat_ptr_volatile_var");
	typeTestbed.checkType("float_ptr_volatile_var","float * volatile");
	typeTestbed.checkValue("float_ptr_volatile_var","&float_var");
    }
    public void test_double_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testdouble_var");
	typeTestbed.checkType("double_var","double");
	typeTestbed.checkValue("double_var","1.797693E308");
    }
    public void test_const_double_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testconst_double_var");
	typeTestbed.checkType("const_double_var","const double");
	typeTestbed.checkValue("const_double_var","2.225074E-308");
    }
    public void test_volatile_double_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testvolatile_double_var");
	typeTestbed.checkType("volatile_double_var","volatile double");
	typeTestbed.checkValue("volatile_double_var","2.225074E-308");
    }
    public void test_ptr_double_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_double_var");
	typeTestbed.checkType("ptr_double_var","double *");
	typeTestbed.checkValue("ptr_double_var","&double_var");
    }
    public void test_ptr_const_double_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_const_double_var");
	typeTestbed.checkType("ptr_const_double_var","const double *");
	typeTestbed.checkValue("ptr_const_double_var","&double_var");
    }
    public void test_double_ptr_const_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testdouble_ptr_const_var");
	typeTestbed.checkType("double_ptr_const_var","double * const");
	typeTestbed.checkValue("double_ptr_const_var","&double_var");
    }
    public void test_ptr_volatile_double_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_volatile_double_var");
	typeTestbed.checkType("ptr_volatile_double_var","volatile double *");
	typeTestbed.checkValue("ptr_volatile_double_var","&double_var");
    }
    public void test_double_ptr_volatile_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testdouble_ptr_volatile_var");
	typeTestbed.checkType("double_ptr_volatile_var","double * volatile");
	typeTestbed.checkValue("double_ptr_volatile_var","&double_var");
    }
    public void test_arr_char () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_char");
	typeTestbed.checkType("arr_char","char [2]");
	typeTestbed.checkValue("arr_char","\"!~\"");
    }
    public void test_arr_arr_char () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_char");
	typeTestbed.checkType("arr_arr_char","char [2][2]");
	typeTestbed.checkValue("arr_arr_char","{{\"!~\"},{\"!~\"}}");
    }
    public void test_arr_arr_arr_char () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_char");
	typeTestbed.checkType("arr_arr_arr_char","char [2][2][2]");
	typeTestbed.checkValue("arr_arr_arr_char","{{{\"!~\"},{\"!~\"}},{{\"!~\"},{\"!~\"}}}");
    }
    public void test_arr_ptr_arr_arr_char () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_arr_arr_char");
	typeTestbed.checkType("arr_ptr_arr_arr_char","char * [2]");
	typeTestbed.checkValue("arr_ptr_arr_arr_char","{arr_arr_char[0],arr_arr_char[1]}");
    }
    public void test_ptr_arr_char () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_char");
	typeTestbed.checkType("ptr_arr_char","char (*)[2]");
	typeTestbed.checkValue("ptr_arr_char","&arr_char");
    }
    public void test_arr_short_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_short_int");
	typeTestbed.checkType("arr_short_int","short int [2]");
	typeTestbed.checkValue("arr_short_int","{-32767,32767}");
    }
    public void test_arr_arr_short_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_short_int");
	typeTestbed.checkType("arr_arr_short_int","short int [2][2]");
	typeTestbed.checkValue("arr_arr_short_int","{{-32767,32767},{-32767,32767}}");
    }
    public void test_arr_arr_arr_short_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_short_int");
	typeTestbed.checkType("arr_arr_arr_short_int","short int [2][2][2]");
	typeTestbed.checkValue("arr_arr_arr_short_int","{{{-32767,32767},{-32767,32767}},{{-32767,32767},{-32767,32767}}}");
    }
    public void test_arr_arr_arr_arr_short_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_arr_short_int");
	typeTestbed.checkType("arr_arr_arr_arr_short_int","short int [3][2][3][2]");
	typeTestbed.checkValue("arr_arr_arr_arr_short_int","{{{{-32767,32767},{-32767,32767},{-32767,32767}},{{-32767,32767},{-32767,32767},{-32767,32767}}},{{{-32767,32767},{-32767,32767},{-32767,32767}},{{-32767,32767},{-32767,32767},{-32767,32767}}},{{{-32767,32767},{-32767,32767},{-32767,32767}},{{-32767,32767},{-32767,32767},{-32767,32767}}}}");
    }
    public void test_arr_ptr_arr_arr_short_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_arr_arr_short_int");
	typeTestbed.checkType("arr_ptr_arr_arr_short_int","short int * [2]");
	typeTestbed.checkValue("arr_ptr_arr_arr_short_int","{arr_arr_short_int[0],arr_arr_short_int[1]}");
    }
    public void test_ptr_arr_short_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_short_int");
	typeTestbed.checkType("ptr_arr_short_int","short int (*)[2]");
	typeTestbed.checkValue("ptr_arr_short_int","&arr_short_int");
    }
    public void test_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_int");
	typeTestbed.checkType("arr_int","int [2]");
	typeTestbed.checkValue("arr_int","{-65536,65536}");
    }
    public void test_arr_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_int");
	typeTestbed.checkType("arr_arr_int","int [2][2]");
	typeTestbed.checkValue("arr_arr_int","{{-65536,65536},{-65536,65536}}");
    }
    public void test_arr_arr_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_int");
	typeTestbed.checkType("arr_arr_arr_int","int [2][2][2]");
	typeTestbed.checkValue("arr_arr_arr_int","{{{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536}}}");
    }
    public void test_arr_arr_arr_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_arr_int");
	typeTestbed.checkType("arr_arr_arr_arr_int","int [3][2][3][2]");
	typeTestbed.checkValue("arr_arr_arr_arr_int","{{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}},{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}},{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}}}");
    }
    public void test_arr_ptr_arr_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_arr_arr_int");
	typeTestbed.checkType("arr_ptr_arr_arr_int","int * [2]");
	typeTestbed.checkValue("arr_ptr_arr_arr_int","{arr_arr_int[0],arr_arr_int[1]}");
    }
    public void test_ptr_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_int");
	typeTestbed.checkType("ptr_arr_int","int (*)[2]");
	typeTestbed.checkValue("ptr_arr_int","&arr_int");
    }
    public void test_arr_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_long_int");
	typeTestbed.checkType("arr_long_int","long int [2]");
	typeTestbed.checkValue("arr_long_int","{-65536,65536}");
    }
    public void test_arr_arr_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_long_int");
	typeTestbed.checkType("arr_arr_long_int","long int [2][2]");
	typeTestbed.checkValue("arr_arr_long_int","{{-65536,65536},{-65536,65536}}");
    }
    public void test_arr_arr_arr_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_long_int");
	typeTestbed.checkType("arr_arr_arr_long_int","long int [2][2][2]");
	typeTestbed.checkValue("arr_arr_arr_long_int","{{{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536}}}");
    }
    public void test_arr_arr_arr_arr_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_arr_long_int");
	typeTestbed.checkType("arr_arr_arr_arr_long_int","long int [3][2][3][2]");
	typeTestbed.checkValue("arr_arr_arr_arr_long_int","{{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}},{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}},{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}}}");
    }
    public void test_arr_ptr_arr_arr_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_arr_arr_long_int");
	typeTestbed.checkType("arr_ptr_arr_arr_long_int","long int * [2]");
	typeTestbed.checkValue("arr_ptr_arr_arr_long_int","{arr_arr_long_int[0],arr_arr_long_int[1]}");
    }
    public void test_ptr_arr_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_long_int");
	typeTestbed.checkType("ptr_arr_long_int","long int (*)[2]");
	typeTestbed.checkValue("ptr_arr_long_int","&arr_long_int");
    }
    public void test_arr_long_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_long_long_int");
	typeTestbed.checkType("arr_long_long_int","long long int [2]");
	typeTestbed.checkValue("arr_long_long_int","{-65536,65536}");
    }
    public void test_arr_arr_long_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_long_long_int");
	typeTestbed.checkType("arr_arr_long_long_int","long long int [2][2]");
	typeTestbed.checkValue("arr_arr_long_long_int","{{-65536,65536},{-65536,65536}}");
    }
    public void test_arr_arr_arr_long_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_long_long_int");
	typeTestbed.checkType("arr_arr_arr_long_long_int","long long int [2][2][2]");
	typeTestbed.checkValue("arr_arr_arr_long_long_int","{{{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536}}}");
    }
    public void test_arr_arr_arr_arr_long_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_arr_long_long_int");
	typeTestbed.checkType("arr_arr_arr_arr_long_long_int","long long int [3][2][3][2]");
	typeTestbed.checkValue("arr_arr_arr_arr_long_long_int","{{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}},{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}},{{{-65536,65536},{-65536,65536},{-65536,65536}},{{-65536,65536},{-65536,65536},{-65536,65536}}}}");
    }
    public void test_arr_ptr_arr_arr_long_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_arr_arr_long_long_int");
	typeTestbed.checkType("arr_ptr_arr_arr_long_long_int","long long int * [2]");
	typeTestbed.checkValue("arr_ptr_arr_arr_long_long_int","{arr_arr_long_long_int[0],arr_arr_long_long_int[1]}");
    }
    public void test_ptr_arr_long_long_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_long_long_int");
	typeTestbed.checkType("ptr_arr_long_long_int","long long int (*)[2]");
	typeTestbed.checkValue("ptr_arr_long_long_int","&arr_long_long_int");
    }
    public void test_arr_float () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_float");
	typeTestbed.checkType("arr_float","float [2]");
	typeTestbed.checkValue("arr_float","{1.1754939E-38,3.402823E38}");
    }
    public void test_arr_arr_float () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_float");
	typeTestbed.checkType("arr_arr_float","float [2][2]");
	typeTestbed.checkValue("arr_arr_float","{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}}");
    }
    public void test_arr_arr_arr_float () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_float");
	typeTestbed.checkType("arr_arr_arr_float","float [2][2][2]");
	typeTestbed.checkValue("arr_arr_arr_float","{{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}},{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}}}");
    }
    public void test_arr_arr_arr_arr_float () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_arr_float");
	typeTestbed.checkType("arr_arr_arr_arr_float","float [3][2][3][2]");
	typeTestbed.checkValue("arr_arr_arr_arr_float","{{{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}},{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}}},{{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}},{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}}},{{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}},{{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38},{1.1754939E-38,3.402823E38}}}}");
    }
    public void test_arr_ptr_arr_arr_float () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_arr_arr_float");
	typeTestbed.checkType("arr_ptr_arr_arr_float","float * [2]");
	typeTestbed.checkValue("arr_ptr_arr_arr_float","{arr_arr_float[0],arr_arr_float[1]}");
    }
    public void test_ptr_arr_float () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_float");
	typeTestbed.checkType("ptr_arr_float","float (*)[2]");
	typeTestbed.checkValue("ptr_arr_float","&arr_float");
    }
    public void test_arr_double () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_double");
	typeTestbed.checkType("arr_double","double [2]");
	typeTestbed.checkValue("arr_double","{2.225074E-308,1.797693E308}");
    }
    public void test_arr_arr_double () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_double");
	typeTestbed.checkType("arr_arr_double","double [2][2]");
	typeTestbed.checkValue("arr_arr_double","{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}}");
    }
    public void test_arr_arr_arr_double () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_double");
	typeTestbed.checkType("arr_arr_arr_double","double [2][2][2]");
	typeTestbed.checkValue("arr_arr_arr_double","{{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}},{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}}}");
    }
    public void test_arr_arr_arr_arr_double () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_arr_arr_double");
	typeTestbed.checkType("arr_arr_arr_arr_double","double [3][2][3][2]");
	typeTestbed.checkValue("arr_arr_arr_arr_double","{{{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}},{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}}},{{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}},{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}}},{{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}},{{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308},{2.225074E-308,1.797693E308}}}}");
    }
    public void test_arr_ptr_arr_arr_double () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_arr_arr_double");
	typeTestbed.checkType("arr_ptr_arr_arr_double","double * [2]");
	typeTestbed.checkValue("arr_ptr_arr_arr_double","{arr_arr_double[0],arr_arr_double[1]}");
    }
    public void test_ptr_arr_double () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_double");
	typeTestbed.checkType("ptr_arr_double","double (*)[2]");
	typeTestbed.checkValue("ptr_arr_double","&arr_double");
    }
    public void test_arr_struct () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_struct");
	typeTestbed.checkType("arr_struct","struct {\n  int int_var;\n} [2]");
	typeTestbed.checkValue("arr_struct","{{\n  int_var=1,\n},{\n  int_var=2,\n}}");
    }
    public void test_arr_union () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_union");
	typeTestbed.checkType("arr_union","union {\n  int int_var;\n  float fl;\n} [2]");
	typeTestbed.checkValue("arr_union","{{\n  int_var=1,\n  fl=1.4012985E-45,\n},{\n  int_var=1073741824,\n  fl=2.0,\n}}");
    }
    public void test_arr_arr_struct () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_struct");
	typeTestbed.checkType("arr_arr_struct","struct {\n  int int_var;\n} [2][2]");
	typeTestbed.checkValue("arr_arr_struct","{{{\n  int_var=1,\n},{\n  int_var=2,\n}},{{\n  int_var=3,\n},{\n  int_var=4,\n}}}");
    }
    public void test_arr_arr_union () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_union");
	typeTestbed.checkType("arr_arr_union","union {\n  int int_var;\n  float float_var;\n} [2][2]");
	typeTestbed.checkValue("arr_arr_union","{{{\n  int_var=1,\n  float_var=1.4012985E-45,\n},{\n  int_var=2,\n  float_var=2.802597E-45,\n}},{{\n  int_var=3,\n  float_var=4.2038954E-45,\n},{\n  int_var=4,\n  float_var=5.605194E-45,\n}}}");
    }
    public void test_arr_arr_ptr () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_arr_ptr");
	typeTestbed.checkType("arr_arr_ptr","int * [2][2]");
	typeTestbed.checkValue("arr_arr_ptr","{{&one,&two},{&three,&four}}");
    }
    public void test_arr_struct_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_struct_arr_int");
	typeTestbed.checkType("arr_struct_arr_int","struct {\n  int arr_int[2];\n} [2]");
	typeTestbed.checkValue("arr_struct_arr_int","{{\n  arr_int={1,2},\n},{\n  arr_int={3,4},\n}}");
    }
    public void test_arr_struct_struct () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_struct_struct");
	typeTestbed.checkType("arr_struct_struct","struct {\n  struct {\n    int int_var;\n  } struct_a;\n} [2]");
	typeTestbed.checkValue("arr_struct_struct","{{\n  struct_a={\n    int_var=1,\n  },\n},{\n  struct_a={\n    int_var=2,\n  },\n}}");
    }
    public void test_arr_struct_union () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_struct_union");
	typeTestbed.checkType("arr_struct_union","struct {\n  union {\n    int int_var;\n    float float_var;\n  } struct_a;\n} [2]");
	typeTestbed.checkValue("arr_struct_union","{{\n  struct_a={\n    int_var=1,\n    float_var=1.4012985E-45,\n  },\n},{\n  struct_a={\n    int_var=2,\n    float_var=2.802597E-45,\n  },\n}}");
    }
    public void test_arr_struct_ptr () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_struct_ptr");
	typeTestbed.checkType("arr_struct_ptr","struct {\n  int *ptr;\n} [2]");
	typeTestbed.checkValue("arr_struct_ptr","{{&one},{&two}}");
    }
    public void test_arr_union_arr_int () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_union_arr_int");
	typeTestbed.checkType("arr_union_arr_int","union {\n  int arr_int[2];\n  float arr_float[2];\n} [2]");
	typeTestbed.checkValue("arr_union_arr_int","{{\n  arr_int={1,2},\n  arr_float={1.4012985E-45,2.802597E-45},\n},{\n  arr_int={3,4},\n  arr_float={4.2038954E-45,5.605194E-45},\n}}");
    }
    public void test_arr_union_struct () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_union_struct");
	typeTestbed.checkType("arr_union_struct","union {\n  struct {\n    int int_var;\n  } struct_a;\n} [2]");
	typeTestbed.checkValue("arr_union_struct","{{\n  struct_a={\n    int_var=1,\n  },\n},{\n  struct_a={\n    int_var=2,\n  },\n}}");
    }
    public void test_arr_union_union () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_union_union");
	typeTestbed.checkType("arr_union_union","union {\n  union {\n    int int_var;\n  } union_a;\n} [2]");
	typeTestbed.checkValue("arr_union_union","{{\n  union_a={\n    int_var=1,\n  },\n},{\n  union_a={\n    int_var=2,\n  },\n}}");
    }
    public void test_arr_union_ptr () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_union_ptr");
	typeTestbed.checkType("arr_union_ptr","union {\n  int *ptr;\n} [2]");
	typeTestbed.checkValue("arr_union_ptr","{{&one}, {&two}}");
    }
    public void test_arr_ptr_struct () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_struct");
	typeTestbed.checkType("arr_ptr_struct","struct {\n  int int_var;\n} * [2]");
	typeTestbed.checkValue("arr_ptr_struct","");
    }
    public void test_arr_ptr_union () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_union");
	typeTestbed.checkType("arr_ptr_union","union {\n  int int_var;\n  float float_var;\n} * [2]");
	typeTestbed.checkValue("arr_ptr_union","");
    }
    public void test_arr_ptr_ptr () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testarr_ptr_ptr");
	typeTestbed.checkType("arr_ptr_ptr","int ** [2]");
	typeTestbed.checkValue("arr_ptr_ptr","");
    }
    public void test_ptr_arr_struct () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_struct");
	typeTestbed.checkType("ptr_arr_struct","struct {\n  int int_var;\n} (*)[2]");
	typeTestbed.checkValue("ptr_arr_struct","");
    }
    public void test_ptr_arr_union () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_union");
	typeTestbed.checkType("ptr_arr_union","union {\n  int int_var;\n} (*)[2]");
	typeTestbed.checkValue("ptr_arr_union","");
    }
    public void test_ptr_arr_ptr () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_arr_ptr");
	typeTestbed.checkType("ptr_arr_ptr","int * (*)[2]");
	typeTestbed.checkValue("ptr_arr_ptr","");
    }
    public void test_bitfields_small_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testbitfields_small_var");
	typeTestbed.checkType("bitfields_small_var","struct {\n  unsigned int bit1_0:1;\n  unsigned int bit1_1:1;\n  char char_2;\n  unsigned int bit1_6:1;\n  unsigned int bit1_7:1;\n  char char_8;\n  unsigned int bit1_9:1;\n  unsigned int bit1_10:1;\n}");
	typeTestbed.checkValue("bitfields_small_var","{\n  bit1_0=1,\n  bit1_1=0,\n  char_2='a',\n  bit1_6=1,\n  bit1_7=0,\n  char_8='z',\n  bit1_9=1,\n  bit1_10=0,\n}");
    }
    public void test_bitfields_bit_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testbitfields_bit_var");
	typeTestbed.checkType("bitfields_bit_var","struct {\n  unsigned char char_0;\n  int bit1_4:1;\n  unsigned int bit1_5:1;\n  int bit2_6:2;\n  unsigned int bit2_8:2;\n  int bit3_10:3;\n  unsigned int bit3_13:3;\n  int bit9_16:9;\n  unsigned int bit9_25:9;\n  char char_34;\n}");
	typeTestbed.checkValue("bitfields_bit_var","{\n  char_0='A',\n  bit1_4=-1,\n  bit1_5=1,\n  bit2_6=1,\n  bit2_8=3,\n  bit3_10=3,\n  bit3_13=7,\n  bit9_16=255,\n  bit9_25=511,\n  char_34='Z',\n}");
    }
    public void test_struct_arr_short_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_arr_short_var");
	typeTestbed.checkType("struct_arr_short_var","struct {\n  short int arr_short[2];\n}");
	typeTestbed.checkValue("struct_arr_short_var","{\n  arr_short={1,2},\n}");
    }
    public void test_struct_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_struct_var");
	typeTestbed.checkType("struct_struct_var","struct {\n  struct {\n    int int_var;\n  } struct_a;\n}");
	typeTestbed.checkValue("struct_struct_var","{\n  struct_a={\n    int_var=1,\n  },\n}");
    }
    public void test_struct_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_union_var");
	typeTestbed.checkType("struct_union_var","struct {\n  union {\n    int int_var;\n  } union_a;\n}");
	typeTestbed.checkValue("struct_union_var","{\n  union_a={\n    int_var=1,\n  },\n}");
    }
    public void test_struct_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_ptr_var");
	typeTestbed.checkType("struct_ptr_var","struct {\n  int *ptr_int;\n}");
	typeTestbed.checkValue("struct_ptr_var","{&one}");
    }
    public void test_union_arr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_arr_var");
	typeTestbed.checkType("union_arr_var","union {\n  int arr_int[4];\n  float arr_float[4];\n}");
	typeTestbed.checkValue("union_arr_var","{\n  arr_int={1,2,3,4},\n  arr_float={1.4012985E-45,2.802597E-45,4.2038954E-45,5.605194E-45},\n}");
    }
    public void test_union_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_struct_var");
	typeTestbed.checkType("union_struct_var","union {\n  struct {\n    int int_var;\n  } struct_a;\n}");
	typeTestbed.checkValue("union_struct_var","{\n  struct_a={\n    int_var=1,\n  },\n}");
    }
    public void test_union_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_union_var");
	typeTestbed.checkType("union_union_var","union {\n  union {\n    int int_var;\n    float float_var;\n  } union_a;\n}");
	typeTestbed.checkValue("union_union_var","{\n  union_a={\n    int_var=1,\n    float_var=1.4012985E-45,\n  },\n}");
    }
    public void test_union_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_ptr_var");
	typeTestbed.checkType("union_ptr_var","union {\n  int *ptr_int;\n}");
	typeTestbed.checkValue("union_ptr_var","{&one}");
    }
    public void test_ptr_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_struct_var");
	typeTestbed.checkType("ptr_struct_var","struct {\n  int int_var;\n} *");
	typeTestbed.checkValue("ptr_struct_var","");
    }
    public void test_ptr_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_union_var");
	typeTestbed.checkType("ptr_union_var","union {\n  int int_var;\n} *");
	typeTestbed.checkValue("ptr_union_var","");
    }
    public void test_struct_arr_arr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_arr_arr_int_var");
	typeTestbed.checkType("struct_arr_arr_int_var","struct {\n  int arr_arr_int[2][2];\n}");
	typeTestbed.checkValue("struct_arr_arr_int_var","{\n  arr_arr_int={{1,2},{3,4}},\n}");
    }
    public void test_struct_arr_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_arr_struct_var");
	typeTestbed.checkType("struct_arr_struct_var","struct {\n  struct {\n    int int_var;\n  } arr_struct[2];\n}");
	typeTestbed.checkValue("struct_arr_struct_var","{\n  arr_struct={{\n    int_var=1,\n  },{\n    int_var=2,\n  }},\n}");
    }
    public void test_struct_arr_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_arr_union_var");
	typeTestbed.checkType("struct_arr_union_var","struct {\n  union {\n    int int_var;\n  } arr_union[4];\n}");
	typeTestbed.checkValue("struct_arr_union_var","{\n  arr_union={{\n    int_var=1,\n  },{\n    int_var=2,\n  },{\n    int_var=0,\n  },{\n    int_var=0,\n  }},\n}");
    }
    public void test_struct_arr_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_arr_ptr_var");
	typeTestbed.checkType("struct_arr_ptr_var","struct {\n  int * arr_ptr[2];\n}");
	typeTestbed.checkValue("struct_arr_ptr_var","{{&one, &two}}");
    }
    public void test_struct_struct_arr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_struct_arr_int_var");
	typeTestbed.checkType("struct_struct_arr_int_var","struct {\n  struct {\n    int arr_int[2];\n  } struct_arr;\n}");
	typeTestbed.checkValue("struct_struct_arr_int_var","{\n  struct_arr={\n    arr_int={1,2},\n  },\n}");
    }
    public void test_struct_struct_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_struct_struct_var");
	typeTestbed.checkType("struct_struct_struct_var","struct {\n  struct {\n    struct {\n      int int_var;\n    } struct_a;\n  } struct_struct;\n}");
	typeTestbed.checkValue("struct_struct_struct_var","{\n  struct_struct={\n    struct_a={\n      int_var=1,\n    },\n  },\n}");
    }
    public void test_struct_struct_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_struct_union_var");
	typeTestbed.checkType("struct_struct_union_var","struct {\n  struct {\n    union {\n      char int_var;\n    } union_a;\n  } struct_union;\n}");
	typeTestbed.checkValue("struct_struct_union_var","{\n  struct_union={\n    union_a={\n      int_var='~',\n    },\n  },\n}");
    }
    public void test_struct_struct_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_struct_ptr_var");
	typeTestbed.checkType("struct_struct_ptr_var","struct {\n  struct {\n    int *ptr_int;\n  } sp;\n}");
	typeTestbed.checkValue("struct_struct_ptr_var","{{&three}}");
    }
    public void test_struct_union_arr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_union_arr_var");
	typeTestbed.checkType("struct_union_arr_var","struct {\n  union {\n    int arr_int[4];\n  } union_arr_int;\n}");
	typeTestbed.checkValue("struct_union_arr_var","{\n  union_arr_int={\n    arr_int={1,2,3,4},\n  },\n}");
    }
    public void test_struct_union_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_union_struct_var");
	typeTestbed.checkType("struct_union_struct_var","struct {\n  union {\n    struct {\n      int int_var;\n    } struct_a;\n  } union_struct;\n}");
	typeTestbed.checkValue("struct_union_struct_var","{\n  union_struct={\n    struct_a={\n      int_var=1,\n    },\n  },\n}");
    }
    public void test_struct_union_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_union_union_var");
	typeTestbed.checkType("struct_union_union_var","struct {\n  union {\n    union {\n      long int int_var;\n      float float_var;\n    } union_a;\n  } union_union;\n}");
	typeTestbed.checkValue("struct_union_union_var","{\n  union_union={\n    union_a={\n      int_var=1,\n      float_var=1.4012985E-45,\n    },\n  },\n}");
    }
    public void test_struct_union_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_union_ptr_var");
	typeTestbed.checkType("struct_union_ptr_var","struct {\n  union {\n    int *ptr_int;\n  } union_ptr;\n}");
	typeTestbed.checkValue("struct_union_ptr_var","{{&four}}");
    }
    public void test_struct_ptr_arr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_ptr_arr_var");
	typeTestbed.checkType("struct_ptr_arr_var","struct {\n  int (*ptr_arr)[4];\n}");
	typeTestbed.checkValue("struct_ptr_arr_var","");
    }
    public void test_struct_ptr_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_ptr_struct_var");
	typeTestbed.checkType("struct_ptr_struct_var","struct {\n  struct {\n    int int_var;\n  } *ptr_struct;\n}");
	typeTestbed.checkValue("struct_ptr_struct_var","");
    }
    public void test_struct_ptr_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_ptr_union_var");
	typeTestbed.checkType("struct_ptr_union_var","struct {\n  union {\n    int int_var;\n  } *ptr_union;\n}");
	typeTestbed.checkValue("struct_ptr_union_var","");
    }
    public void test_struct_ptr_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "teststruct_ptr_ptr_var");
	typeTestbed.checkType("struct_ptr_ptr_var","struct {\n  int **ptr_ptr;\n}");
	typeTestbed.checkValue("struct_ptr_ptr_var","");
    }
    public void test_union_arr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_arr_int_var");
	typeTestbed.checkType("union_arr_int_var","union {\n  int arr_int[2][2];\n  float arr_float[2][2];\n}");
	typeTestbed.checkValue("union_arr_int_var","{\n  arr_int={{1,2},{3,4}},\n  arr_float={{1.4012985E-45,2.802597E-45},{4.2038954E-45,5.605194E-45}},\n}");
    }
    public void test_union_arr_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_arr_struct_var");
	typeTestbed.checkType("union_arr_struct_var","union {\n  struct {\n    int int_var;\n  } arr_struct[2];\n}");
	typeTestbed.checkValue("union_arr_struct_var","{\n  arr_struct={{\n    int_var=1,\n  },{\n    int_var=0,\n  }},\n}");
    }
    public void test_union_arr_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_arr_union_var");
	typeTestbed.checkType("union_arr_union_var","union {\n  union {\n    int int_var;\n    float float_var;\n  } arr_union[2];\n}");
	typeTestbed.checkValue("union_arr_union_var","{\n  arr_union={{\n    int_var=1,\n    float_var=1.4012985E-45,\n  },{\n    int_var=0,\n    float_var=0.0,\n  }},\n}");
    }
    public void test_union_arr_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_arr_ptr_var");
	typeTestbed.checkType("union_arr_ptr_var","union {\n  int * arr_ptr[4];\n}");
	typeTestbed.checkValue("union_arr_ptr_var","{{&one}}");
    }
    public void test_union_struct_arr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_struct_arr_int_var");
	typeTestbed.checkType("union_struct_arr_int_var","union {\n  struct {\n    int arr_int[4];\n  } struct_arr;\n}");
	typeTestbed.checkValue("union_struct_arr_int_var","{\n  struct_arr={\n    arr_int={1,2,3,4},\n  },\n}");
    }
    public void test_union_struct_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_struct_struct_var");
	typeTestbed.checkType("union_struct_struct_var","union {\n  struct {\n    struct {\n      int int_var;\n    } struct_a;\n  } struct_b;\n}");
	typeTestbed.checkValue("union_struct_struct_var","{\n  struct_b={\n    struct_a={\n      int_var=1,\n    },\n  },\n}");
    }
    public void test_union_struct_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_struct_union_var");
	typeTestbed.checkType("union_struct_union_var","union {\n  struct {\n    union {\n      int int_var;\n      float float_var;\n    } union_a;\n  } struct_a;\n}");
	typeTestbed.checkValue("union_struct_union_var","{\n  struct_a={\n    union_a={\n      int_var=1,\n      float_var=1.4012985E-45,\n    },\n  },\n}");
    }
    public void test_union_struct_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_struct_ptr_var");
	typeTestbed.checkType("union_struct_ptr_var","union {\n  struct {\n    int *ptr_int;\n  } struct_ptr;\n}");
	typeTestbed.checkValue("union_struct_ptr_var","{{&one}}");
    }
    public void test_union_union_arr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_union_arr_int_var");
	typeTestbed.checkType("union_union_arr_int_var","union {\n  union {\n    long long int arr_int[4];\n  } union_arr;\n}");
	typeTestbed.checkValue("union_union_arr_int_var","{\n  union_arr={\n    arr_int={1,2,3,4},\n  },\n}");
    }
    public void test_union_union_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_union_struct_var");
	typeTestbed.checkType("union_union_struct_var","union {\n  union {\n    struct {\n      long long int int_var;\n    } struct_a;\n  } union_a;\n}");
	typeTestbed.checkValue("union_union_struct_var","{\n  union_a={\n    struct_a={\n      int_var=1,\n    },\n  },\n}");
    }
    public void test_union_union_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_union_union_var");
	typeTestbed.checkType("union_union_union_var","union {\n  union {\n    union {\n      int int_var;\n      float float_var;\n    } union_a;\n  } union_b;\n}");
	typeTestbed.checkValue("union_union_union_var","{\n  union_b={\n    union_a={\n      int_var=1,\n      float_var=1.4012985E-45,\n    },\n  },\n}");
    }
    public void test_union_union_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_union_ptr_var");
	typeTestbed.checkType("union_union_ptr_var","union {\n  union {\n    int *ptr_int;\n    float *ptr_float;\n  } union_ptr;\n}");
	typeTestbed.checkValue("union_union_ptr_var","{{&one}}");
    }
    public void test_union_ptr_arr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_ptr_arr_var");
	typeTestbed.checkType("union_ptr_arr_var","union {\n  int (*ptr_arr)[4];\n}");
	typeTestbed.checkValue("union_ptr_arr_var","");
    }
    public void test_union_ptr_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_ptr_struct_var");
	typeTestbed.checkType("union_ptr_struct_var","union {\n  struct {\n    int int_var;\n  } *ptr_struct;\n}");
	typeTestbed.checkValue("union_ptr_struct_var","");
    }
    public void test_union_ptr_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_ptr_union_var");
	typeTestbed.checkType("union_ptr_union_var","union {\n  union {\n    int int_var;\n  } *ptr_union;\n}");
	typeTestbed.checkValue("union_ptr_union_var","");
    }
    public void test_union_ptr_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testunion_ptr_ptr_var");
	typeTestbed.checkType("union_ptr_ptr_var","union {\n  int **ptr_ptr;\n}");
	typeTestbed.checkValue("union_ptr_ptr_var","");
    }
    public void test_ptr_struct_arr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_struct_arr_int_var");
	typeTestbed.checkType("ptr_struct_arr_int_var","struct {\n  int arr_int[4];\n} *");
	typeTestbed.checkValue("ptr_struct_arr_int_var","");
    }
    public void test_ptr_struct_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_struct_struct_var");
	typeTestbed.checkType("ptr_struct_struct_var","struct {\n  struct {\n    int int_var;\n  } struct_a;\n} *");
	typeTestbed.checkValue("ptr_struct_struct_var","");
    }
    public void test_ptr_struct_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_struct_union_var");
	typeTestbed.checkType("ptr_struct_union_var","struct {\n  union {\n    int int_var;\n  } union_a;\n} *");
	typeTestbed.checkValue("ptr_struct_union_var","");
    }
    public void test_ptr_struct_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_struct_ptr_var");
	typeTestbed.checkType("ptr_struct_ptr_var","struct {\n  int *ptr_int;\n} *");
	typeTestbed.checkValue("ptr_struct_ptr_var","");
    }
    public void test_ptr_union_arr_int_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_union_arr_int_var");
	typeTestbed.checkType("ptr_union_arr_int_var","union {\n  int arr_int[4];\n} *");
	typeTestbed.checkValue("ptr_union_arr_int_var","");
    }
    public void test_ptr_union_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_union_struct_var");
	typeTestbed.checkType("ptr_union_struct_var","union {\n  struct {\n    int int_var;\n  } struct_a;\n} *");
	typeTestbed.checkValue("ptr_union_struct_var","");
    }
    public void test_ptr_union_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_union_union_var");
	typeTestbed.checkType("ptr_union_union_var","union {\n  union {\n    int int_var;\n  } union_a;\n} *");
	typeTestbed.checkValue("ptr_union_union_var","");
    }
    public void test_ptr_union_ptr_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_union_ptr_var");
	typeTestbed.checkType("ptr_union_ptr_var","union {\n  int *ptr_int;\n} *");
	typeTestbed.checkValue("ptr_union_ptr_var","");
    }
    public void test_ptr_ptr_struct_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_ptr_struct_var");
	typeTestbed.checkType("ptr_ptr_struct_var","struct {\n  int int_var;\n} **");
	typeTestbed.checkValue("ptr_ptr_struct_var","");
    }
    public void test_ptr_ptr_union_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_ptr_union_var");
	typeTestbed.checkType("ptr_ptr_union_var","union {\n  int int_var;\n} **");
	typeTestbed.checkValue("ptr_ptr_union_var","");
    }
    public void test_primary_colors_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testprimary_colors_var");
	typeTestbed.checkType("primary_colors_var","enum  {\n  red = 0,\n  green = 1,\n  blue = 2\n}");
	typeTestbed.checkValue("primary_colors_var","red");
    }
    public void test_rainbow_colors_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testrainbow_colors_var");
	typeTestbed.checkType("rainbow_colors_var","enum colors {\n  orange = 0,\n  yellow = 1,\n  violet = 2,\n  indigo = 3\n}");
	typeTestbed.checkValue("rainbow_colors_var","orange");
    }
    public void test_usa_cars_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testusa_cars_var");
	typeTestbed.checkType("usa_cars_var","enum  {\n  chevy = 0,\n  dodge = 44,\n  ford = 55\n}");
	typeTestbed.checkValue("usa_cars_var","chevy");
    }
    public void test_sports_cars_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testsports_cars_var");
	typeTestbed.checkType("sports_cars_var","enum cars {\n  bmw = 0,\n  mercedes = 1,\n  porsche = 2\n}");
	typeTestbed.checkValue("sports_cars_var","bmw");
    }
    public void test_ptr_void_type_var_var () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-entry", "testptr_void_type_var_var");
	typeTestbed.checkType("ptr_void_type_var_var","void_type *");
	typeTestbed.checkValue("ptr_void_type_var_var","");
    }
    public void test_class_accessibility () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-class", "testclass_accessibility");
	typeTestbed.checkType("class_accessibility","class class_accessibility {\n private:\n  long int long_var;\n  int int_var;\n public:\n  short int short_var;\n protected:\n  float float_var;\n public:\n  void set_int_var (int);\n}");
	typeTestbed.checkValue("class_accessibility","");
    }
    public void test_class_inherited () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-class", "testclass_inherited");
	typeTestbed.checkType("class_inherited","class Type {\n  struct Base1 {\n    const char *msg;\n    void Base1 (const char *);\n    void ~Base1 ();\n  } ;\n  struct Base2 {\n    const char *msg;\n    void Base2 (const char *);\n    void ~Base2 ();\n  } ;\n private:\n  const char *note;\n public:\n  void Type (const char *,const char *,const char *);\n  void ~Type ();\n}");
	typeTestbed.checkValue("class_inherited","");
    }
    public void test_class_inherited_new () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-class", "testclass_inherited_new");
	typeTestbed.checkType("class_inherited_new","class Type {\n  struct Base1 {\n    const char *msg;\n    void Base1 (const char *);\n    void ~Base1 ();\n  } ;\n  struct Base2 {\n    const char *msg;\n    void Base2 (const char *);\n    void ~Base2 ();\n  } ;\n private:\n  const char *note;\n public:\n  void Type (const char *,const char *,const char *);\n  void ~Type ();\n}");
	typeTestbed.checkValue("class_inherited_new","");
    }
    public void test_class_template () {
	TypeTestbed typeTestbed = new TypeTestbed("funit-type-class", "testclass_template");
	typeTestbed.checkType("class_template","class Derived {\n  struct Base3 {\n    int (**_vptr.Base3) ();\n    void Base3 (void * const &);\n    void Base3 ();\n    char do_this (char);\n    short int do_this (short int);\n    int do_this (int);\n    float do_this (float);\n    void ~Base3 ();\n  } ;\n  void Derived (void * const &);\n  void Derived ();\n  char do_this (char);\n  short int do_this (short int);\n  int do_this (int);\n  float do_this (float);\n  void ~Derived ();\n private:\n  char do_this_impl<char> (char);\n  short int do_this_impl<short int> (short int);\n  int do_this_impl<int> (int);\n  float do_this_impl<float> (float);\n}");
	typeTestbed.checkValue("class_template","");
    }
}
