// -*- C++ -*-

// This file is part of the program FRYSK.
//
// Copyright 2006, 2007, 2008, Red Hat Inc.
//
// FRYSK is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// FRYSK is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with FRYSK; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
// 
// In addition, as a special exception, Red Hat, Inc. gives You the
// additional right to link the code of FRYSK with code not covered
// under the GNU General Public License ("Non-GPL Code") and to
// distribute linked combinations including the two, subject to the
// limitations in this paragraph. Non-GPL Code permitted under this
// exception must only link to the code of FRYSK through those well
// defined interfaces identified in the file named EXCEPTION found in
// the source code files (the "Approved Interfaces"). The files of
// Non-GPL Code may instantiate templates or use macros or inline
// functions from the Approved Interfaces without causing the
// resulting work to be covered by the GNU General Public
// License. Only Red Hat, Inc. may make changes or additions to the
// list of Approved Interfaces. You must obey the GNU General Public
// License in all respects for all of the FRYSK code and other code
// used in conjunction with FRYSK except the Non-GPL Code covered by
// this exception. If you modify this file, you may extend this
// exception to your version of the file, but you are not obligated to
// do so. If you do not wish to provide this exception without
// modification, you must delete this exception statement from your
// version and license this file solely under the GPL without
// exception.

#include <gcj/cni.h>

#include <java/lang/String.h>
#include <java/io/File.h>
#include "frysk/config/Config.h"

// Why not use autoconf's config.h?

// Because config.h does not expand variables the way Makefile and
// SUBST_SED do.

static java::io::File*
asFile (const char* path)
{
  return new java::io::File (JvNewStringUTF (path)->intern ());
}

static java::io::File*
asFile (jstring prefix, const char *suffix)
{
  return new java::io::File (prefix, JvNewStringUTF (suffix)->intern ());
}

jstring
frysk::config::Config::getRootSrcDir ()
{
  return JvNewStringUTF ("/usr/src/debug/frysk-0.0.1.2008.03.19.rh1/");
}

jstring
frysk::config::Config::getAbsRootSrcDir ()
{
  return JvNewStringUTF ("/usr/src/debug/frysk-0.0.1.2008.03.19.rh1/");
}

jint
frysk::config::Config::getWordSize ()
{
  return sizeof (long) * 8;
}

jstring
frysk::config::Config::getTargetCpuXXX ()
{
  return JvNewStringUTF ("i686");
}

frysk::config::Config*
frysk::config::Config::createInstallConfig ()
{
  frysk::config::Config* config = new frysk::config::Config ();
  config->theGladeDir = JvNewStringUTF ("/usr/share/frysk/glade/")->intern ();
  config->theHelpDir = asFile ("/usr/share/gnome/help/frysk/C");
  config->theImagesDir = JvNewStringUTF ("/usr/share/frysk/images")->intern ();
  config->theBinDir = asFile ("/usr/bin");
  config->thePkgDataDir = asFile ("/usr/share/frysk");
  config->thePkgLibDir = asFile ("/usr/lib/frysk");
  switch (sizeof(void*)) {
  case 4:
    config->thePkgLib32Dir = config->thePkgLibDir;
    config->thePkgLib64Dir = NULL;
    break;
  case 8:
    config->thePkgLib32Dir = asFile ("/usr/lib/frysk");
    config->thePkgLib64Dir = config->thePkgLibDir;
    break;
  }
  return config;
}

frysk::config::Config*
frysk::config::Config::createInstallConfig32 ()
{
  frysk::config::Config* config = frysk::config::Config::createInstallConfig();
  switch (sizeof(long))
    {
    case 4:
      // default
      break;
    case 8:
      // Test against 32-bit binaries.
      config->thePkgLibDir = config->thePkgLib32Dir;
      // Disable "native" 32-bit.
      config->thePkgLib32Dir = NULL;
      break;
    default:
      return NULL;
    }
  return config;
}

frysk::config::Config*
frysk::config::Config::createInstallConfig64 ()
{
  frysk::config::Config* config = frysk::config::Config::createInstallConfig();
  switch (sizeof(long))
    {
    case 4:
      // 64-bit target not supported on 32-bit host.
      return NULL;
    case 8:
      // disable 32-on-64 testing.
      config->thePkgLib32Dir = NULL;
      break;
    default:
      return NULL;
    }
  return config;
}

frysk::config::Config*
frysk::config::Config::createBuildConfig (jstring absSrcDir,
				  jstring absBuildDir)
{
  frysk::config::Config* config = new frysk::config::Config ();
  config->theGladeDir = absSrcDir->concat (JvNewStringUTF ("/frysk/gui/gladedir/"))->intern ();
  config->theHelpDir = asFile (absSrcDir, "/frysk/gui/helpdir");
  config->theImagesDir = absSrcDir->concat (JvNewStringUTF ("/frysk/gui/imagesdir"))->intern ();
  config->theBinDir = asFile (absBuildDir, "/frysk/bindir/");
  config->thePkgDataDir = asFile (absBuildDir, "/frysk/pkgdatadir/");
  config->thePkgLibDir = asFile (absBuildDir, "/frysk/pkglibdir/");
  switch (sizeof(void*)) {
  case 4:
    config->thePkgLib32Dir = config->thePkgLibDir;
    config->thePkgLib64Dir = NULL;
    break;
  case 8:
    config->thePkgLib32Dir = asFile (absBuildDir, "/frysk/pkglibdir/arch32/");
    config->thePkgLib64Dir = config->thePkgLibDir;
    break;
  }
  return config;
}

frysk::config::Config*
frysk::config::Config::createBuildConfig32 (jstring absSrcDir,
				    jstring absBuildDir)
{
  frysk::config::Config* config = frysk::config::Config::createBuildConfig(absSrcDir,
							   absBuildDir);
  switch (sizeof(long))
    {
    case 4:
      // default
      break;
    case 8:
      // Test against 32-bit binaries.
      config->thePkgLibDir = config->thePkgLib32Dir;
      // disable cross testing.
      config->thePkgLib32Dir = NULL;
      break;
    default:
      return NULL;
    }
  return config;
}

frysk::config::Config*
frysk::config::Config::createBuildConfig64 (jstring absSrcDir,
				    jstring absBuildDir)
{
  frysk::config::Config* config
    = frysk::config::Config::createBuildConfig(absSrcDir, absBuildDir);
  switch (sizeof(long))
    {
    case 4:
      // 64-bit target not supported on 32-bit host.
      return NULL;
      break;
    case 8:
      // disable cross testing
      config->thePkgLib32Dir = NULL;
      break;
    default:
      return NULL;
    }
  return config;
}
