// This file is part of the program FRYSK.
//
// Copyright 2008, Red Hat Inc.
//
// FRYSK is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// FRYSK is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with FRYSK; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
// 
// In addition, as a special exception, Red Hat, Inc. gives You the
// additional right to link the code of FRYSK with code not covered
// under the GNU General Public License ("Non-GPL Code") and to
// distribute linked combinations including the two, subject to the
// limitations in this paragraph. Non-GPL Code permitted under this
// exception must only link to the code of FRYSK through those well
// defined interfaces identified in the file named EXCEPTION found in
// the source code files (the "Approved Interfaces"). The files of
// Non-GPL Code may instantiate templates or use macros or inline
// functions from the Approved Interfaces without causing the
// resulting work to be covered by the GNU General Public
// License. Only Red Hat, Inc. may make changes or additions to the
// list of Approved Interfaces. You must obey the GNU General Public
// License in all respects for all of the FRYSK code and other code
// used in conjunction with FRYSK except the Non-GPL Code covered by
// this exception. If you modify this file, you may extend this
// exception to your version of the file, but you are not obligated to
// do so. If you do not wish to provide this exception without
// modification, you must delete this exception statement from your
// version and license this file solely under the GPL without
// exception.

package frysk.sys;

/**
 * Unix Host Errors.
 *
 * Do not confuse this with target errors and target error numbers.
 */
public class Errno extends RuntimeException {
    static final long serialVersionUID = 0;

    private final String message;
    private Errno(String message) {
        super(message);
	this.message = message;
    }
    /**
     * Create an error corresponding to errno (use when the more specific
     * errno hasn't available).
     */
    public Errno(int num, String message) {
        this(message + " (errno " + num + ")");         
    }

    /**
     * Return the error message (without any decoration).
     */
    public String getMessage() {
        return message;
    }

    public static class Ebadf extends Errno {
        static final long serialVersionUID = 0;
        Ebadf(String message) {
            super(message);
        }
    }

    public static class Enomem extends Errno {
        static final long serialVersionUID = 0;
        Enomem(String message) {
            super(message);
        }
    }

    public static class Efault extends Errno {
        static final long serialVersionUID = 0;
        Efault(String message) {
            super(message);
        }
    }

    public static class Einval extends Errno {
        static final long serialVersionUID = 0;
        Einval(String message) {
            super(message);
        }
    }

    public static class Esrch extends Errno {
        static final long serialVersionUID = 0;
        Esrch(String message) {
            super(message);
        }
    }

    public static class Echild extends Errno {
        static final long serialVersionUID = 0;
        Echild(String message) {
            super(message);
        }
    }

    public static class Eperm extends Errno {
        static final long serialVersionUID = 0;
        Eperm(String message) {
            super(message);
        }
    }

    public static class Eio extends Errno {
        static final long serialVersionUID = 0;
        Eio(String message) {
            super(message);
        }
    }

    public static class Enoent extends Errno {
        static final long serialVersionUID = 0;
        Enoent(String message) {
            super(message);
        }
    }
}
