// This file is part of the program FRYSK.
//
// Copyright 2005, 2007, 2008, Red Hat Inc.
//
// FRYSK is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// FRYSK is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with FRYSK; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
// 
// In addition, as a special exception, Red Hat, Inc. gives You the
// additional right to link the code of FRYSK with code not covered
// under the GNU General Public License ("Non-GPL Code") and to
// distribute linked combinations including the two, subject to the
// limitations in this paragraph. Non-GPL Code permitted under this
// exception must only link to the code of FRYSK through those well
// defined interfaces identified in the file named EXCEPTION found in
// the source code files (the "Approved Interfaces"). The files of
// Non-GPL Code may instantiate templates or use macros or inline
// functions from the Approved Interfaces without causing the
// resulting work to be covered by the GNU General Public
// License. Only Red Hat, Inc. may make changes or additions to the
// list of Approved Interfaces. You must obey the GNU General Public
// License in all respects for all of the FRYSK code and other code
// used in conjunction with FRYSK except the Non-GPL Code covered by
// this exception. If you modify this file, you may extend this
// exception to your version of the file, but you are not obligated to
// do so. If you do not wish to provide this exception without
// modification, you must delete this exception statement from your
// version and license this file solely under the GPL without
// exception.

#include <sys/types.h>
#include <signal.h>
#include <linux/unistd.h>
#include <errno.h>
#include <stdio.h>
#include <linux.syscall.h>
#include <unistd.h>
#include <alloca.h>

#include <gcj/cni.h>

#include "frysk/sys/Signal.h"
#include "frysk/sys/cni/Errno.hxx"

void
frysk::sys::Signal::tkill(jint tid, jint signum, jstring name) {
  errno = 0;
  if (::syscall (__NR_tkill, tid, signum) < 0)
    throwErrno(errno, "tkill", "task %d, signal %s (%d)",
               (int)tid, ALLOCA_STRING(name), (int)signum);
}

void
frysk::sys::Signal::kill(jint pid, jint signum, jstring name) {
  errno = 0;
  if (::kill (pid, signum) < 0)
    throwErrno(errno, "kill", "process %d, signal %s (%d)",
               (int)pid, ALLOCA_STRING(name), (int)signum);
}

void
frysk::sys::Signal::drain (jint signum) {
//   sigset_t set;
//   sigpending (&set);
//   printf ("Before: %d (%s) pending? %s\n", signum, strsignal (signum),
// 	  sigismember (&set, signum) ? "YES" : "NO");
  struct sigaction oldAct = { };
  struct sigaction newAct = { };
  newAct.sa_handler = SIG_IGN;
  if (::sigaction (signum, &newAct, &oldAct))
    throwErrno (errno, "sigaction", "signal %d - %s", (int)signum,
		strsignal(signum));
  if (::sigaction (signum, &oldAct, NULL))
    throwErrno (errno, "sigaction", "signal %d - %s", (int)signum,
		strsignal(signum));
//   sigpending (&set);
//   printf ("After: %d (%s) pending? %s\n", signum, strsignal (signum),
// 	  sigismember (&set, signum) ? "YES" : "NO");
}

jint
frysk::sys::Signal::nsig() {
  return NSIG;
}

jint
frysk::sys::Signal::rtMin() {
  return SIGRTMIN;
}

jint
frysk::sys::Signal::rtMax() {
  return SIGRTMAX;
}

jint
frysk::sys::Signal::hup() {
#ifdef SIGHUP
    return SIGHUP;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::int_() {
#ifdef SIGINT
    return SIGINT;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::quit() {
#ifdef SIGQUIT
    return SIGQUIT;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::ill() {
#ifdef SIGILL
    return SIGILL;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::abrt() {
#ifdef SIGABRT
    return SIGABRT;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::fpe() {
#ifdef SIGFPE
    return SIGFPE;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::kill() {
#ifdef SIGKILL
    return SIGKILL;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::segv() {
#ifdef SIGSEGV
    return SIGSEGV;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::pipe() {
#ifdef SIGPIPE
    return SIGPIPE;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::alrm() {
#ifdef SIGALRM
    return SIGALRM;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::term() {
#ifdef SIGTERM
    return SIGTERM;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::usr1() {
#ifdef SIGUSR1
    return SIGUSR1;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::usr2() {
#ifdef SIGUSR2
    return SIGUSR2;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::chld() {
#ifdef SIGCHLD
    return SIGCHLD;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::cont() {
#ifdef SIGCONT
    return SIGCONT;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::stop() {
#ifdef SIGSTOP
    return SIGSTOP;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::tstp() {
#ifdef SIGTSTP
    return SIGTSTP;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::ttin() {
#ifdef SIGTTIN
    return SIGTTIN;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::ttou() {
#ifdef SIGTTOU
    return SIGTTOU;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::bus() {
#ifdef SIGBUS
    return SIGBUS;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::poll() {
#ifdef SIGPOLL
    return SIGPOLL;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::prof() {
#ifdef SIGPROF
    return SIGPROF;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::sys() {
#ifdef SIGSYS
    return SIGSYS;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::trap() {
#ifdef SIGTRAP
    return SIGTRAP;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::urg() {
#ifdef SIGURG
    return SIGURG;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::vtalrm() {
#ifdef SIGVTALRM
    return SIGVTALRM;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::xcpu() {
#ifdef SIGXCPU
    return SIGXCPU;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::xfsz() {
#ifdef SIGXFSZ
    return SIGXFSZ;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::iot() {
#ifdef SIGIOT
    return SIGIOT;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::emt() {
#ifdef SIGEMT
    return SIGEMT;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::stkflt() {
#ifdef SIGSTKFLT
    return SIGSTKFLT;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::io() {
#ifdef SIGIO
    return SIGIO;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::cld() {
#ifdef SIGCLD
    return SIGCLD;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::pwr() {
#ifdef SIGPWR
    return SIGPWR;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::info() {
#ifdef SIGINFO
    return SIGINFO;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::lost() {
#ifdef SIGLOST
    return SIGLOST;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::winch() {
#ifdef SIGWINCH
    return SIGWINCH;
#else
    return -1;
#endif
}
jint
frysk::sys::Signal::unused() {
#ifdef SIGUNUSED
    return SIGUNUSED;
#else
    return -1;
#endif
}
