package lib.dwfl;

public class DwOp
  extends Number
  implements Comparable
{
  private final String enumString;
  private final int enumValue;
  private final String enumPrint;
  private final String enumName;
  private DwOp (int value, String string, String print, String name)
  {
    enumString = string;
    enumValue = value;
    enumPrint = print;
    enumName = name;
  }
  /** Return the qualified name of the enum.  */
  public String toString ()
  {
    return enumString;
  }
  /** Return a printable version of the enum.  */
  public String toPrint ()
  {
    return enumPrint;
  }
  /** Return the name of just the enum.  */
  public String toName ()
  {
    return enumName;
  }
  /**
   * Return true if OBJECT has the same value.
   */
  public boolean equals (Object o)
  {
    if (o == null)
      return false;
    if (! (o instanceof DwOp))
      return false;
    return ((DwOp)o).intValue() == this.intValue();
  }
  public int hashCode ()
  {
    return enumValue;
  }
  public int compareTo (Object o)
  {
    DwOp rhs = (DwOp) o; // Can throw - ok.
    return this.enumValue - rhs.enumValue;
  }
  static public final int ADDR_ = 0x03;
  static public final DwOp ADDR = new DwOp (0x03, "DwOp_ADDR", "DW_OP_addr", "ADDR");
  static public final int DEREF_ = 0x06;
  static public final DwOp DEREF = new DwOp (0x06, "DwOp_DEREF", "DW_OP_deref", "DEREF");
  static public final int CONST1U_ = 0x08;
  static public final DwOp CONST1U = new DwOp (0x08, "DwOp_CONST1U", "DW_OP_const1u", "CONST1U");
  static public final int CONST1S_ = 0x09;
  static public final DwOp CONST1S = new DwOp (0x09, "DwOp_CONST1S", "DW_OP_const1s", "CONST1S");
  static public final int CONST2U_ = 0x0a;
  static public final DwOp CONST2U = new DwOp (0x0a, "DwOp_CONST2U", "DW_OP_const2u", "CONST2U");
  static public final int CONST2S_ = 0x0b;
  static public final DwOp CONST2S = new DwOp (0x0b, "DwOp_CONST2S", "DW_OP_const2s", "CONST2S");
  static public final int CONST4U_ = 0x0c;
  static public final DwOp CONST4U = new DwOp (0x0c, "DwOp_CONST4U", "DW_OP_const4u", "CONST4U");
  static public final int CONST4S_ = 0x0d;
  static public final DwOp CONST4S = new DwOp (0x0d, "DwOp_CONST4S", "DW_OP_const4s", "CONST4S");
  static public final int CONST8U_ = 0x0e;
  static public final DwOp CONST8U = new DwOp (0x0e, "DwOp_CONST8U", "DW_OP_const8u", "CONST8U");
  static public final int CONST8S_ = 0x0f;
  static public final DwOp CONST8S = new DwOp (0x0f, "DwOp_CONST8S", "DW_OP_const8s", "CONST8S");
  static public final int CONSTU_ = 0x10;
  static public final DwOp CONSTU = new DwOp (0x10, "DwOp_CONSTU", "DW_OP_constu", "CONSTU");
  static public final int CONSTS_ = 0x11;
  static public final DwOp CONSTS = new DwOp (0x11, "DwOp_CONSTS", "DW_OP_consts", "CONSTS");
  static public final int DUP_ = 0x12;
  static public final DwOp DUP = new DwOp (0x12, "DwOp_DUP", "DW_OP_dup", "DUP");
  static public final int DROP_ = 0x13;
  static public final DwOp DROP = new DwOp (0x13, "DwOp_DROP", "DW_OP_drop", "DROP");
  static public final int OVER_ = 0x14;
  static public final DwOp OVER = new DwOp (0x14, "DwOp_OVER", "DW_OP_over", "OVER");
  static public final int PICK_ = 0x15;
  static public final DwOp PICK = new DwOp (0x15, "DwOp_PICK", "DW_OP_pick", "PICK");
  static public final int SWAP_ = 0x16;
  static public final DwOp SWAP = new DwOp (0x16, "DwOp_SWAP", "DW_OP_swap", "SWAP");
  static public final int ROT_ = 0x17;
  static public final DwOp ROT = new DwOp (0x17, "DwOp_ROT", "DW_OP_rot", "ROT");
  static public final int XDEREF_ = 0x18;
  static public final DwOp XDEREF = new DwOp (0x18, "DwOp_XDEREF", "DW_OP_xderef", "XDEREF");
  static public final int ABS_ = 0x19;
  static public final DwOp ABS = new DwOp (0x19, "DwOp_ABS", "DW_OP_abs", "ABS");
  static public final int AND_ = 0x1a;
  static public final DwOp AND = new DwOp (0x1a, "DwOp_AND", "DW_OP_and", "AND");
  static public final int DIV_ = 0x1b;
  static public final DwOp DIV = new DwOp (0x1b, "DwOp_DIV", "DW_OP_div", "DIV");
  static public final int MINUS_ = 0x1c;
  static public final DwOp MINUS = new DwOp (0x1c, "DwOp_MINUS", "DW_OP_minus", "MINUS");
  static public final int MOD_ = 0x1d;
  static public final DwOp MOD = new DwOp (0x1d, "DwOp_MOD", "DW_OP_mod", "MOD");
  static public final int MUL_ = 0x1e;
  static public final DwOp MUL = new DwOp (0x1e, "DwOp_MUL", "DW_OP_mul", "MUL");
  static public final int NEG_ = 0x1f;
  static public final DwOp NEG = new DwOp (0x1f, "DwOp_NEG", "DW_OP_neg", "NEG");
  static public final int NOT_ = 0x20;
  static public final DwOp NOT = new DwOp (0x20, "DwOp_NOT", "DW_OP_not", "NOT");
  static public final int OR_ = 0x21;
  static public final DwOp OR = new DwOp (0x21, "DwOp_OR", "DW_OP_or", "OR");
  static public final int PLUS_ = 0x22;
  static public final DwOp PLUS = new DwOp (0x22, "DwOp_PLUS", "DW_OP_plus", "PLUS");
  static public final int PLUS_UCONST_ = 0x23;
  static public final DwOp PLUS_UCONST = new DwOp (0x23, "DwOp_PLUS_UCONST", "DW_OP_plus_uconst", "PLUS_UCONST");
  static public final int SHL_ = 0x24;
  static public final DwOp SHL = new DwOp (0x24, "DwOp_SHL", "DW_OP_shl", "SHL");
  static public final int SHR_ = 0x25;
  static public final DwOp SHR = new DwOp (0x25, "DwOp_SHR", "DW_OP_shr", "SHR");
  static public final int SHRA_ = 0x26;
  static public final DwOp SHRA = new DwOp (0x26, "DwOp_SHRA", "DW_OP_shra", "SHRA");
  static public final int XOR_ = 0x27;
  static public final DwOp XOR = new DwOp (0x27, "DwOp_XOR", "DW_OP_xor", "XOR");
  static public final int BRA_ = 0x28;
  static public final DwOp BRA = new DwOp (0x28, "DwOp_BRA", "DW_OP_bra", "BRA");
  static public final int EQ_ = 0x29;
  static public final DwOp EQ = new DwOp (0x29, "DwOp_EQ", "DW_OP_eq", "EQ");
  static public final int GE_ = 0x2a;
  static public final DwOp GE = new DwOp (0x2a, "DwOp_GE", "DW_OP_ge", "GE");
  static public final int GT_ = 0x2b;
  static public final DwOp GT = new DwOp (0x2b, "DwOp_GT", "DW_OP_gt", "GT");
  static public final int LE_ = 0x2c;
  static public final DwOp LE = new DwOp (0x2c, "DwOp_LE", "DW_OP_le", "LE");
  static public final int LT_ = 0x2d;
  static public final DwOp LT = new DwOp (0x2d, "DwOp_LT", "DW_OP_lt", "LT");
  static public final int NE_ = 0x2e;
  static public final DwOp NE = new DwOp (0x2e, "DwOp_NE", "DW_OP_ne", "NE");
  static public final int SKIP_ = 0x2f;
  static public final DwOp SKIP = new DwOp (0x2f, "DwOp_SKIP", "DW_OP_skip", "SKIP");
  static public final int LIT0_ = 0x30;
  static public final DwOp LIT0 = new DwOp (0x30, "DwOp_LIT0", "DW_OP_lit0", "LIT0");
  static public final int LIT1_ = 0x31;
  static public final DwOp LIT1 = new DwOp (0x31, "DwOp_LIT1", "DW_OP_lit1", "LIT1");
  static public final int LIT2_ = 0x32;
  static public final DwOp LIT2 = new DwOp (0x32, "DwOp_LIT2", "DW_OP_lit2", "LIT2");
  static public final int LIT3_ = 0x33;
  static public final DwOp LIT3 = new DwOp (0x33, "DwOp_LIT3", "DW_OP_lit3", "LIT3");
  static public final int LIT4_ = 0x34;
  static public final DwOp LIT4 = new DwOp (0x34, "DwOp_LIT4", "DW_OP_lit4", "LIT4");
  static public final int LIT5_ = 0x35;
  static public final DwOp LIT5 = new DwOp (0x35, "DwOp_LIT5", "DW_OP_lit5", "LIT5");
  static public final int LIT6_ = 0x36;
  static public final DwOp LIT6 = new DwOp (0x36, "DwOp_LIT6", "DW_OP_lit6", "LIT6");
  static public final int LIT7_ = 0x37;
  static public final DwOp LIT7 = new DwOp (0x37, "DwOp_LIT7", "DW_OP_lit7", "LIT7");
  static public final int LIT8_ = 0x38;
  static public final DwOp LIT8 = new DwOp (0x38, "DwOp_LIT8", "DW_OP_lit8", "LIT8");
  static public final int LIT9_ = 0x39;
  static public final DwOp LIT9 = new DwOp (0x39, "DwOp_LIT9", "DW_OP_lit9", "LIT9");
  static public final int LIT10_ = 0x3a;
  static public final DwOp LIT10 = new DwOp (0x3a, "DwOp_LIT10", "DW_OP_lit10", "LIT10");
  static public final int LIT11_ = 0x3b;
  static public final DwOp LIT11 = new DwOp (0x3b, "DwOp_LIT11", "DW_OP_lit11", "LIT11");
  static public final int LIT12_ = 0x3c;
  static public final DwOp LIT12 = new DwOp (0x3c, "DwOp_LIT12", "DW_OP_lit12", "LIT12");
  static public final int LIT13_ = 0x3d;
  static public final DwOp LIT13 = new DwOp (0x3d, "DwOp_LIT13", "DW_OP_lit13", "LIT13");
  static public final int LIT14_ = 0x3e;
  static public final DwOp LIT14 = new DwOp (0x3e, "DwOp_LIT14", "DW_OP_lit14", "LIT14");
  static public final int LIT15_ = 0x3f;
  static public final DwOp LIT15 = new DwOp (0x3f, "DwOp_LIT15", "DW_OP_lit15", "LIT15");
  static public final int LIT16_ = 0x40;
  static public final DwOp LIT16 = new DwOp (0x40, "DwOp_LIT16", "DW_OP_lit16", "LIT16");
  static public final int LIT17_ = 0x41;
  static public final DwOp LIT17 = new DwOp (0x41, "DwOp_LIT17", "DW_OP_lit17", "LIT17");
  static public final int LIT18_ = 0x42;
  static public final DwOp LIT18 = new DwOp (0x42, "DwOp_LIT18", "DW_OP_lit18", "LIT18");
  static public final int LIT19_ = 0x43;
  static public final DwOp LIT19 = new DwOp (0x43, "DwOp_LIT19", "DW_OP_lit19", "LIT19");
  static public final int LIT20_ = 0x44;
  static public final DwOp LIT20 = new DwOp (0x44, "DwOp_LIT20", "DW_OP_lit20", "LIT20");
  static public final int LIT21_ = 0x45;
  static public final DwOp LIT21 = new DwOp (0x45, "DwOp_LIT21", "DW_OP_lit21", "LIT21");
  static public final int LIT22_ = 0x46;
  static public final DwOp LIT22 = new DwOp (0x46, "DwOp_LIT22", "DW_OP_lit22", "LIT22");
  static public final int LIT23_ = 0x47;
  static public final DwOp LIT23 = new DwOp (0x47, "DwOp_LIT23", "DW_OP_lit23", "LIT23");
  static public final int LIT24_ = 0x48;
  static public final DwOp LIT24 = new DwOp (0x48, "DwOp_LIT24", "DW_OP_lit24", "LIT24");
  static public final int LIT25_ = 0x49;
  static public final DwOp LIT25 = new DwOp (0x49, "DwOp_LIT25", "DW_OP_lit25", "LIT25");
  static public final int LIT26_ = 0x4a;
  static public final DwOp LIT26 = new DwOp (0x4a, "DwOp_LIT26", "DW_OP_lit26", "LIT26");
  static public final int LIT27_ = 0x4b;
  static public final DwOp LIT27 = new DwOp (0x4b, "DwOp_LIT27", "DW_OP_lit27", "LIT27");
  static public final int LIT28_ = 0x4c;
  static public final DwOp LIT28 = new DwOp (0x4c, "DwOp_LIT28", "DW_OP_lit28", "LIT28");
  static public final int LIT29_ = 0x4d;
  static public final DwOp LIT29 = new DwOp (0x4d, "DwOp_LIT29", "DW_OP_lit29", "LIT29");
  static public final int LIT30_ = 0x4e;
  static public final DwOp LIT30 = new DwOp (0x4e, "DwOp_LIT30", "DW_OP_lit30", "LIT30");
  static public final int LIT31_ = 0x4f;
  static public final DwOp LIT31 = new DwOp (0x4f, "DwOp_LIT31", "DW_OP_lit31", "LIT31");
  static public final int REG0_ = 0x50;
  static public final DwOp REG0 = new DwOp (0x50, "DwOp_REG0", "DW_OP_reg0", "REG0");
  static public final int REG1_ = 0x51;
  static public final DwOp REG1 = new DwOp (0x51, "DwOp_REG1", "DW_OP_reg1", "REG1");
  static public final int REG2_ = 0x52;
  static public final DwOp REG2 = new DwOp (0x52, "DwOp_REG2", "DW_OP_reg2", "REG2");
  static public final int REG3_ = 0x53;
  static public final DwOp REG3 = new DwOp (0x53, "DwOp_REG3", "DW_OP_reg3", "REG3");
  static public final int REG4_ = 0x54;
  static public final DwOp REG4 = new DwOp (0x54, "DwOp_REG4", "DW_OP_reg4", "REG4");
  static public final int REG5_ = 0x55;
  static public final DwOp REG5 = new DwOp (0x55, "DwOp_REG5", "DW_OP_reg5", "REG5");
  static public final int REG6_ = 0x56;
  static public final DwOp REG6 = new DwOp (0x56, "DwOp_REG6", "DW_OP_reg6", "REG6");
  static public final int REG7_ = 0x57;
  static public final DwOp REG7 = new DwOp (0x57, "DwOp_REG7", "DW_OP_reg7", "REG7");
  static public final int REG8_ = 0x58;
  static public final DwOp REG8 = new DwOp (0x58, "DwOp_REG8", "DW_OP_reg8", "REG8");
  static public final int REG9_ = 0x59;
  static public final DwOp REG9 = new DwOp (0x59, "DwOp_REG9", "DW_OP_reg9", "REG9");
  static public final int REG10_ = 0x5a;
  static public final DwOp REG10 = new DwOp (0x5a, "DwOp_REG10", "DW_OP_reg10", "REG10");
  static public final int REG11_ = 0x5b;
  static public final DwOp REG11 = new DwOp (0x5b, "DwOp_REG11", "DW_OP_reg11", "REG11");
  static public final int REG12_ = 0x5c;
  static public final DwOp REG12 = new DwOp (0x5c, "DwOp_REG12", "DW_OP_reg12", "REG12");
  static public final int REG13_ = 0x5d;
  static public final DwOp REG13 = new DwOp (0x5d, "DwOp_REG13", "DW_OP_reg13", "REG13");
  static public final int REG14_ = 0x5e;
  static public final DwOp REG14 = new DwOp (0x5e, "DwOp_REG14", "DW_OP_reg14", "REG14");
  static public final int REG15_ = 0x5f;
  static public final DwOp REG15 = new DwOp (0x5f, "DwOp_REG15", "DW_OP_reg15", "REG15");
  static public final int REG16_ = 0x60;
  static public final DwOp REG16 = new DwOp (0x60, "DwOp_REG16", "DW_OP_reg16", "REG16");
  static public final int REG17_ = 0x61;
  static public final DwOp REG17 = new DwOp (0x61, "DwOp_REG17", "DW_OP_reg17", "REG17");
  static public final int REG18_ = 0x62;
  static public final DwOp REG18 = new DwOp (0x62, "DwOp_REG18", "DW_OP_reg18", "REG18");
  static public final int REG19_ = 0x63;
  static public final DwOp REG19 = new DwOp (0x63, "DwOp_REG19", "DW_OP_reg19", "REG19");
  static public final int REG20_ = 0x64;
  static public final DwOp REG20 = new DwOp (0x64, "DwOp_REG20", "DW_OP_reg20", "REG20");
  static public final int REG21_ = 0x65;
  static public final DwOp REG21 = new DwOp (0x65, "DwOp_REG21", "DW_OP_reg21", "REG21");
  static public final int REG22_ = 0x66;
  static public final DwOp REG22 = new DwOp (0x66, "DwOp_REG22", "DW_OP_reg22", "REG22");
  static public final int REG23_ = 0x67;
  static public final DwOp REG23 = new DwOp (0x67, "DwOp_REG23", "DW_OP_reg23", "REG23");
  static public final int REG24_ = 0x68;
  static public final DwOp REG24 = new DwOp (0x68, "DwOp_REG24", "DW_OP_reg24", "REG24");
  static public final int REG25_ = 0x69;
  static public final DwOp REG25 = new DwOp (0x69, "DwOp_REG25", "DW_OP_reg25", "REG25");
  static public final int REG26_ = 0x6a;
  static public final DwOp REG26 = new DwOp (0x6a, "DwOp_REG26", "DW_OP_reg26", "REG26");
  static public final int REG27_ = 0x6b;
  static public final DwOp REG27 = new DwOp (0x6b, "DwOp_REG27", "DW_OP_reg27", "REG27");
  static public final int REG28_ = 0x6c;
  static public final DwOp REG28 = new DwOp (0x6c, "DwOp_REG28", "DW_OP_reg28", "REG28");
  static public final int REG29_ = 0x6d;
  static public final DwOp REG29 = new DwOp (0x6d, "DwOp_REG29", "DW_OP_reg29", "REG29");
  static public final int REG30_ = 0x6e;
  static public final DwOp REG30 = new DwOp (0x6e, "DwOp_REG30", "DW_OP_reg30", "REG30");
  static public final int REG31_ = 0x6f;
  static public final DwOp REG31 = new DwOp (0x6f, "DwOp_REG31", "DW_OP_reg31", "REG31");
  static public final int BREG0_ = 0x70;
  static public final DwOp BREG0 = new DwOp (0x70, "DwOp_BREG0", "DW_OP_breg0", "BREG0");
  static public final int BREG1_ = 0x71;
  static public final DwOp BREG1 = new DwOp (0x71, "DwOp_BREG1", "DW_OP_breg1", "BREG1");
  static public final int BREG2_ = 0x72;
  static public final DwOp BREG2 = new DwOp (0x72, "DwOp_BREG2", "DW_OP_breg2", "BREG2");
  static public final int BREG3_ = 0x73;
  static public final DwOp BREG3 = new DwOp (0x73, "DwOp_BREG3", "DW_OP_breg3", "BREG3");
  static public final int BREG4_ = 0x74;
  static public final DwOp BREG4 = new DwOp (0x74, "DwOp_BREG4", "DW_OP_breg4", "BREG4");
  static public final int BREG5_ = 0x75;
  static public final DwOp BREG5 = new DwOp (0x75, "DwOp_BREG5", "DW_OP_breg5", "BREG5");
  static public final int BREG6_ = 0x76;
  static public final DwOp BREG6 = new DwOp (0x76, "DwOp_BREG6", "DW_OP_breg6", "BREG6");
  static public final int BREG7_ = 0x77;
  static public final DwOp BREG7 = new DwOp (0x77, "DwOp_BREG7", "DW_OP_breg7", "BREG7");
  static public final int BREG8_ = 0x78;
  static public final DwOp BREG8 = new DwOp (0x78, "DwOp_BREG8", "DW_OP_breg8", "BREG8");
  static public final int BREG9_ = 0x79;
  static public final DwOp BREG9 = new DwOp (0x79, "DwOp_BREG9", "DW_OP_breg9", "BREG9");
  static public final int BREG10_ = 0x7a;
  static public final DwOp BREG10 = new DwOp (0x7a, "DwOp_BREG10", "DW_OP_breg10", "BREG10");
  static public final int BREG11_ = 0x7b;
  static public final DwOp BREG11 = new DwOp (0x7b, "DwOp_BREG11", "DW_OP_breg11", "BREG11");
  static public final int BREG12_ = 0x7c;
  static public final DwOp BREG12 = new DwOp (0x7c, "DwOp_BREG12", "DW_OP_breg12", "BREG12");
  static public final int BREG13_ = 0x7d;
  static public final DwOp BREG13 = new DwOp (0x7d, "DwOp_BREG13", "DW_OP_breg13", "BREG13");
  static public final int BREG14_ = 0x7e;
  static public final DwOp BREG14 = new DwOp (0x7e, "DwOp_BREG14", "DW_OP_breg14", "BREG14");
  static public final int BREG15_ = 0x7f;
  static public final DwOp BREG15 = new DwOp (0x7f, "DwOp_BREG15", "DW_OP_breg15", "BREG15");
  static public final int BREG16_ = 0x80;
  static public final DwOp BREG16 = new DwOp (0x80, "DwOp_BREG16", "DW_OP_breg16", "BREG16");
  static public final int BREG17_ = 0x81;
  static public final DwOp BREG17 = new DwOp (0x81, "DwOp_BREG17", "DW_OP_breg17", "BREG17");
  static public final int BREG18_ = 0x82;
  static public final DwOp BREG18 = new DwOp (0x82, "DwOp_BREG18", "DW_OP_breg18", "BREG18");
  static public final int BREG19_ = 0x83;
  static public final DwOp BREG19 = new DwOp (0x83, "DwOp_BREG19", "DW_OP_breg19", "BREG19");
  static public final int BREG20_ = 0x84;
  static public final DwOp BREG20 = new DwOp (0x84, "DwOp_BREG20", "DW_OP_breg20", "BREG20");
  static public final int BREG21_ = 0x85;
  static public final DwOp BREG21 = new DwOp (0x85, "DwOp_BREG21", "DW_OP_breg21", "BREG21");
  static public final int BREG22_ = 0x86;
  static public final DwOp BREG22 = new DwOp (0x86, "DwOp_BREG22", "DW_OP_breg22", "BREG22");
  static public final int BREG23_ = 0x87;
  static public final DwOp BREG23 = new DwOp (0x87, "DwOp_BREG23", "DW_OP_breg23", "BREG23");
  static public final int BREG24_ = 0x88;
  static public final DwOp BREG24 = new DwOp (0x88, "DwOp_BREG24", "DW_OP_breg24", "BREG24");
  static public final int BREG25_ = 0x89;
  static public final DwOp BREG25 = new DwOp (0x89, "DwOp_BREG25", "DW_OP_breg25", "BREG25");
  static public final int BREG26_ = 0x8a;
  static public final DwOp BREG26 = new DwOp (0x8a, "DwOp_BREG26", "DW_OP_breg26", "BREG26");
  static public final int BREG27_ = 0x8b;
  static public final DwOp BREG27 = new DwOp (0x8b, "DwOp_BREG27", "DW_OP_breg27", "BREG27");
  static public final int BREG28_ = 0x8c;
  static public final DwOp BREG28 = new DwOp (0x8c, "DwOp_BREG28", "DW_OP_breg28", "BREG28");
  static public final int BREG29_ = 0x8d;
  static public final DwOp BREG29 = new DwOp (0x8d, "DwOp_BREG29", "DW_OP_breg29", "BREG29");
  static public final int BREG30_ = 0x8e;
  static public final DwOp BREG30 = new DwOp (0x8e, "DwOp_BREG30", "DW_OP_breg30", "BREG30");
  static public final int BREG31_ = 0x8f;
  static public final DwOp BREG31 = new DwOp (0x8f, "DwOp_BREG31", "DW_OP_breg31", "BREG31");
  static public final int REGX_ = 0x90;
  static public final DwOp REGX = new DwOp (0x90, "DwOp_REGX", "DW_OP_regx", "REGX");
  static public final int FBREG_ = 0x91;
  static public final DwOp FBREG = new DwOp (0x91, "DwOp_FBREG", "DW_OP_fbreg", "FBREG");
  static public final int BREGX_ = 0x92;
  static public final DwOp BREGX = new DwOp (0x92, "DwOp_BREGX", "DW_OP_bregx", "BREGX");
  static public final int PIECE_ = 0x93;
  static public final DwOp PIECE = new DwOp (0x93, "DwOp_PIECE", "DW_OP_piece", "PIECE");
  static public final int DEREF_SIZE_ = 0x94;
  static public final DwOp DEREF_SIZE = new DwOp (0x94, "DwOp_DEREF_SIZE", "DW_OP_deref_size", "DEREF_SIZE");
  static public final int XDEREF_SIZE_ = 0x95;
  static public final DwOp XDEREF_SIZE = new DwOp (0x95, "DwOp_XDEREF_SIZE", "DW_OP_xderef_size", "XDEREF_SIZE");
  static public final int NOP_ = 0x96;
  static public final DwOp NOP = new DwOp (0x96, "DwOp_NOP", "DW_OP_nop", "NOP");
  static public final int PUSH_OBJECT_ADDRESS_ = 0x97;
  static public final DwOp PUSH_OBJECT_ADDRESS = new DwOp (0x97, "DwOp_PUSH_OBJECT_ADDRESS", "DW_OP_push_object_address", "PUSH_OBJECT_ADDRESS");
  static public final int CALL2_ = 0x98;
  static public final DwOp CALL2 = new DwOp (0x98, "DwOp_CALL2", "DW_OP_call2", "CALL2");
  static public final int CALL4_ = 0x99;
  static public final DwOp CALL4 = new DwOp (0x99, "DwOp_CALL4", "DW_OP_call4", "CALL4");
  static public final int CALL_REF_ = 0x9a;
  static public final DwOp CALL_REF = new DwOp (0x9a, "DwOp_CALL_REF", "DW_OP_call_ref", "CALL_REF");
  static public final int FORM_TLS_ADDRESS_ = 0x9b;
  static public final DwOp FORM_TLS_ADDRESS = new DwOp (0x9b, "DwOp_FORM_TLS_ADDRESS", "DW_OP_form_tls_address", "FORM_TLS_ADDRESS");
  static public final int CALL_FRAME_CFA_ = 0x9c;
  static public final DwOp CALL_FRAME_CFA = new DwOp (0x9c, "DwOp_CALL_FRAME_CFA", "DW_OP_call_frame_cfa", "CALL_FRAME_CFA");
  static public final int BIT_PIECE_ = 0x9d;
  static public final DwOp BIT_PIECE = new DwOp (0x9d, "DwOp_BIT_PIECE", "DW_OP_bit_piece", "BIT_PIECE");
  static public final int LO_USER_ = 0xe0;
  static public final DwOp LO_USER = new DwOp (0xe0, "DwOp_LO_USER", "DW_OP_lo_user", "LO_USER");
  static public final int HI_USER_ = 0xff;
  static public final DwOp HI_USER = new DwOp (0xff, "DwOp_HI_USER", "DW_OP_hi_user", "HI_USER");

  /** Create a HashMap containing all the DwOp elements.  */
  private static java.util.Map getMap ()
  {
    java.util.Map map;
    map = new java.util.HashMap ();
    map.put (ADDR.enumName, ADDR);
    map.put (DEREF.enumName, DEREF);
    map.put (CONST1U.enumName, CONST1U);
    map.put (CONST1S.enumName, CONST1S);
    map.put (CONST2U.enumName, CONST2U);
    map.put (CONST2S.enumName, CONST2S);
    map.put (CONST4U.enumName, CONST4U);
    map.put (CONST4S.enumName, CONST4S);
    map.put (CONST8U.enumName, CONST8U);
    map.put (CONST8S.enumName, CONST8S);
    map.put (CONSTU.enumName, CONSTU);
    map.put (CONSTS.enumName, CONSTS);
    map.put (DUP.enumName, DUP);
    map.put (DROP.enumName, DROP);
    map.put (OVER.enumName, OVER);
    map.put (PICK.enumName, PICK);
    map.put (SWAP.enumName, SWAP);
    map.put (ROT.enumName, ROT);
    map.put (XDEREF.enumName, XDEREF);
    map.put (ABS.enumName, ABS);
    map.put (AND.enumName, AND);
    map.put (DIV.enumName, DIV);
    map.put (MINUS.enumName, MINUS);
    map.put (MOD.enumName, MOD);
    map.put (MUL.enumName, MUL);
    map.put (NEG.enumName, NEG);
    map.put (NOT.enumName, NOT);
    map.put (OR.enumName, OR);
    map.put (PLUS.enumName, PLUS);
    map.put (PLUS_UCONST.enumName, PLUS_UCONST);
    map.put (SHL.enumName, SHL);
    map.put (SHR.enumName, SHR);
    map.put (SHRA.enumName, SHRA);
    map.put (XOR.enumName, XOR);
    map.put (BRA.enumName, BRA);
    map.put (EQ.enumName, EQ);
    map.put (GE.enumName, GE);
    map.put (GT.enumName, GT);
    map.put (LE.enumName, LE);
    map.put (LT.enumName, LT);
    map.put (NE.enumName, NE);
    map.put (SKIP.enumName, SKIP);
    map.put (LIT0.enumName, LIT0);
    map.put (LIT1.enumName, LIT1);
    map.put (LIT2.enumName, LIT2);
    map.put (LIT3.enumName, LIT3);
    map.put (LIT4.enumName, LIT4);
    map.put (LIT5.enumName, LIT5);
    map.put (LIT6.enumName, LIT6);
    map.put (LIT7.enumName, LIT7);
    map.put (LIT8.enumName, LIT8);
    map.put (LIT9.enumName, LIT9);
    map.put (LIT10.enumName, LIT10);
    map.put (LIT11.enumName, LIT11);
    map.put (LIT12.enumName, LIT12);
    map.put (LIT13.enumName, LIT13);
    map.put (LIT14.enumName, LIT14);
    map.put (LIT15.enumName, LIT15);
    map.put (LIT16.enumName, LIT16);
    map.put (LIT17.enumName, LIT17);
    map.put (LIT18.enumName, LIT18);
    map.put (LIT19.enumName, LIT19);
    map.put (LIT20.enumName, LIT20);
    map.put (LIT21.enumName, LIT21);
    map.put (LIT22.enumName, LIT22);
    map.put (LIT23.enumName, LIT23);
    map.put (LIT24.enumName, LIT24);
    map.put (LIT25.enumName, LIT25);
    map.put (LIT26.enumName, LIT26);
    map.put (LIT27.enumName, LIT27);
    map.put (LIT28.enumName, LIT28);
    map.put (LIT29.enumName, LIT29);
    map.put (LIT30.enumName, LIT30);
    map.put (LIT31.enumName, LIT31);
    map.put (REG0.enumName, REG0);
    map.put (REG1.enumName, REG1);
    map.put (REG2.enumName, REG2);
    map.put (REG3.enumName, REG3);
    map.put (REG4.enumName, REG4);
    map.put (REG5.enumName, REG5);
    map.put (REG6.enumName, REG6);
    map.put (REG7.enumName, REG7);
    map.put (REG8.enumName, REG8);
    map.put (REG9.enumName, REG9);
    map.put (REG10.enumName, REG10);
    map.put (REG11.enumName, REG11);
    map.put (REG12.enumName, REG12);
    map.put (REG13.enumName, REG13);
    map.put (REG14.enumName, REG14);
    map.put (REG15.enumName, REG15);
    map.put (REG16.enumName, REG16);
    map.put (REG17.enumName, REG17);
    map.put (REG18.enumName, REG18);
    map.put (REG19.enumName, REG19);
    map.put (REG20.enumName, REG20);
    map.put (REG21.enumName, REG21);
    map.put (REG22.enumName, REG22);
    map.put (REG23.enumName, REG23);
    map.put (REG24.enumName, REG24);
    map.put (REG25.enumName, REG25);
    map.put (REG26.enumName, REG26);
    map.put (REG27.enumName, REG27);
    map.put (REG28.enumName, REG28);
    map.put (REG29.enumName, REG29);
    map.put (REG30.enumName, REG30);
    map.put (REG31.enumName, REG31);
    map.put (BREG0.enumName, BREG0);
    map.put (BREG1.enumName, BREG1);
    map.put (BREG2.enumName, BREG2);
    map.put (BREG3.enumName, BREG3);
    map.put (BREG4.enumName, BREG4);
    map.put (BREG5.enumName, BREG5);
    map.put (BREG6.enumName, BREG6);
    map.put (BREG7.enumName, BREG7);
    map.put (BREG8.enumName, BREG8);
    map.put (BREG9.enumName, BREG9);
    map.put (BREG10.enumName, BREG10);
    map.put (BREG11.enumName, BREG11);
    map.put (BREG12.enumName, BREG12);
    map.put (BREG13.enumName, BREG13);
    map.put (BREG14.enumName, BREG14);
    map.put (BREG15.enumName, BREG15);
    map.put (BREG16.enumName, BREG16);
    map.put (BREG17.enumName, BREG17);
    map.put (BREG18.enumName, BREG18);
    map.put (BREG19.enumName, BREG19);
    map.put (BREG20.enumName, BREG20);
    map.put (BREG21.enumName, BREG21);
    map.put (BREG22.enumName, BREG22);
    map.put (BREG23.enumName, BREG23);
    map.put (BREG24.enumName, BREG24);
    map.put (BREG25.enumName, BREG25);
    map.put (BREG26.enumName, BREG26);
    map.put (BREG27.enumName, BREG27);
    map.put (BREG28.enumName, BREG28);
    map.put (BREG29.enumName, BREG29);
    map.put (BREG30.enumName, BREG30);
    map.put (BREG31.enumName, BREG31);
    map.put (REGX.enumName, REGX);
    map.put (FBREG.enumName, FBREG);
    map.put (BREGX.enumName, BREGX);
    map.put (PIECE.enumName, PIECE);
    map.put (DEREF_SIZE.enumName, DEREF_SIZE);
    map.put (XDEREF_SIZE.enumName, XDEREF_SIZE);
    map.put (NOP.enumName, NOP);
    map.put (PUSH_OBJECT_ADDRESS.enumName, PUSH_OBJECT_ADDRESS);
    map.put (CALL2.enumName, CALL2);
    map.put (CALL4.enumName, CALL4);
    map.put (CALL_REF.enumName, CALL_REF);
    map.put (FORM_TLS_ADDRESS.enumName, FORM_TLS_ADDRESS);
    map.put (CALL_FRAME_CFA.enumName, CALL_FRAME_CFA);
    map.put (BIT_PIECE.enumName, BIT_PIECE);
    map.put (LO_USER.enumName, LO_USER);
    map.put (HI_USER.enumName, HI_USER);
    return map;
  }
  private static java.util.Map enumMap = getMap ();

  /** Return the DwOp object that matches the string.  */
  public static DwOp valueOf (String string)
  {
    return (DwOp)enumMap.get (string);
  }

  /** Return the DwOp object that matches the integer.  */
  public static DwOp valueOf (long i)
  {
    switch ((int)i) {
    case ADDR_: return ADDR;
    case DEREF_: return DEREF;
    case CONST1U_: return CONST1U;
    case CONST1S_: return CONST1S;
    case CONST2U_: return CONST2U;
    case CONST2S_: return CONST2S;
    case CONST4U_: return CONST4U;
    case CONST4S_: return CONST4S;
    case CONST8U_: return CONST8U;
    case CONST8S_: return CONST8S;
    case CONSTU_: return CONSTU;
    case CONSTS_: return CONSTS;
    case DUP_: return DUP;
    case DROP_: return DROP;
    case OVER_: return OVER;
    case PICK_: return PICK;
    case SWAP_: return SWAP;
    case ROT_: return ROT;
    case XDEREF_: return XDEREF;
    case ABS_: return ABS;
    case AND_: return AND;
    case DIV_: return DIV;
    case MINUS_: return MINUS;
    case MOD_: return MOD;
    case MUL_: return MUL;
    case NEG_: return NEG;
    case NOT_: return NOT;
    case OR_: return OR;
    case PLUS_: return PLUS;
    case PLUS_UCONST_: return PLUS_UCONST;
    case SHL_: return SHL;
    case SHR_: return SHR;
    case SHRA_: return SHRA;
    case XOR_: return XOR;
    case BRA_: return BRA;
    case EQ_: return EQ;
    case GE_: return GE;
    case GT_: return GT;
    case LE_: return LE;
    case LT_: return LT;
    case NE_: return NE;
    case SKIP_: return SKIP;
    case LIT0_: return LIT0;
    case LIT1_: return LIT1;
    case LIT2_: return LIT2;
    case LIT3_: return LIT3;
    case LIT4_: return LIT4;
    case LIT5_: return LIT5;
    case LIT6_: return LIT6;
    case LIT7_: return LIT7;
    case LIT8_: return LIT8;
    case LIT9_: return LIT9;
    case LIT10_: return LIT10;
    case LIT11_: return LIT11;
    case LIT12_: return LIT12;
    case LIT13_: return LIT13;
    case LIT14_: return LIT14;
    case LIT15_: return LIT15;
    case LIT16_: return LIT16;
    case LIT17_: return LIT17;
    case LIT18_: return LIT18;
    case LIT19_: return LIT19;
    case LIT20_: return LIT20;
    case LIT21_: return LIT21;
    case LIT22_: return LIT22;
    case LIT23_: return LIT23;
    case LIT24_: return LIT24;
    case LIT25_: return LIT25;
    case LIT26_: return LIT26;
    case LIT27_: return LIT27;
    case LIT28_: return LIT28;
    case LIT29_: return LIT29;
    case LIT30_: return LIT30;
    case LIT31_: return LIT31;
    case REG0_: return REG0;
    case REG1_: return REG1;
    case REG2_: return REG2;
    case REG3_: return REG3;
    case REG4_: return REG4;
    case REG5_: return REG5;
    case REG6_: return REG6;
    case REG7_: return REG7;
    case REG8_: return REG8;
    case REG9_: return REG9;
    case REG10_: return REG10;
    case REG11_: return REG11;
    case REG12_: return REG12;
    case REG13_: return REG13;
    case REG14_: return REG14;
    case REG15_: return REG15;
    case REG16_: return REG16;
    case REG17_: return REG17;
    case REG18_: return REG18;
    case REG19_: return REG19;
    case REG20_: return REG20;
    case REG21_: return REG21;
    case REG22_: return REG22;
    case REG23_: return REG23;
    case REG24_: return REG24;
    case REG25_: return REG25;
    case REG26_: return REG26;
    case REG27_: return REG27;
    case REG28_: return REG28;
    case REG29_: return REG29;
    case REG30_: return REG30;
    case REG31_: return REG31;
    case BREG0_: return BREG0;
    case BREG1_: return BREG1;
    case BREG2_: return BREG2;
    case BREG3_: return BREG3;
    case BREG4_: return BREG4;
    case BREG5_: return BREG5;
    case BREG6_: return BREG6;
    case BREG7_: return BREG7;
    case BREG8_: return BREG8;
    case BREG9_: return BREG9;
    case BREG10_: return BREG10;
    case BREG11_: return BREG11;
    case BREG12_: return BREG12;
    case BREG13_: return BREG13;
    case BREG14_: return BREG14;
    case BREG15_: return BREG15;
    case BREG16_: return BREG16;
    case BREG17_: return BREG17;
    case BREG18_: return BREG18;
    case BREG19_: return BREG19;
    case BREG20_: return BREG20;
    case BREG21_: return BREG21;
    case BREG22_: return BREG22;
    case BREG23_: return BREG23;
    case BREG24_: return BREG24;
    case BREG25_: return BREG25;
    case BREG26_: return BREG26;
    case BREG27_: return BREG27;
    case BREG28_: return BREG28;
    case BREG29_: return BREG29;
    case BREG30_: return BREG30;
    case BREG31_: return BREG31;
    case REGX_: return REGX;
    case FBREG_: return FBREG;
    case BREGX_: return BREGX;
    case PIECE_: return PIECE;
    case DEREF_SIZE_: return DEREF_SIZE;
    case XDEREF_SIZE_: return XDEREF_SIZE;
    case NOP_: return NOP;
    case PUSH_OBJECT_ADDRESS_: return PUSH_OBJECT_ADDRESS;
    case CALL2_: return CALL2;
    case CALL4_: return CALL4;
    case CALL_REF_: return CALL_REF;
    case FORM_TLS_ADDRESS_: return FORM_TLS_ADDRESS;
    case CALL_FRAME_CFA_: return CALL_FRAME_CFA;
    case BIT_PIECE_: return BIT_PIECE;
    case LO_USER_: return LO_USER;
    case HI_USER_: return HI_USER;
    default: return null;
    }
  }

  /** Return an array of all the DwOp elements.  */
  public static DwOp[] values ()
  {
    return (DwOp[]) enumMap.values ().toArray (new DwOp[0]);
  }

  /**
   * Returns the full underscore delimited name of the
   * field corresponding to the value I.
   */
  static public String toString (long i)
  {
    DwOp c = valueOf (i);
    if (c == null)
      return "_0x" + Long.toHexString (i);
    else
      return c.toString ();
  }

  /**
   * Returns the printable (or user readable) name for the
   * field corresponding to the value I.
   */
  static public String toPrintString (long i)
  {
    DwOp c = valueOf (i);
    if (c == null)
      return "_0x" + Long.toHexString (i);
    else
      return c.toPrint ();
  }

  /**
   * Returns the printable (or user readable) name for the
   * field corresponding to the value I, or DEF is there
   * is no such field.
   */
  static public String toPrintString (long i, String def)
  {
    DwOp c = valueOf (i);
    if (c == null)
      return def;
    else
      return c.toPrint ();
  }

  /**
   * Returns just the name part of the num corresponding to I.
   */
  static public String toName (long i)
  {
    DwOp c = valueOf (i);
    if (c == null)
      return "_0x" + Long.toHexString (i);
    else
      return c.toName ();
  }

  /**
   * Returns just the name part of the num corresponding to I,
   * or DEF is there is no such field.
   */
  static public String toName (long i, String def)
  {
    DwOp c = valueOf (i);
    if (c == null)
      return def;
    else
      return c.toName ();
  }

  /**
   * Return the equivalent of the enum.
   */
  public int intValue () {
    return enumValue;
  }

  /**
   * Return the equivalent of the enum.
   */
  public long longValue () {
    return enumValue;
  }

  /**
   * Return the equivalent of the enum.
   */
  public float floatValue () {
    return enumValue;
  }

  /**
   * Return the equivalent of the enum.
   */
  public double doubleValue () {
    return enumValue;
  }

  static final long serialVersionUID = 0;
}
