package lib.dwfl;

public class DwTag
  extends Number
  implements Comparable
{
  private final String enumString;
  private final int enumValue;
  private final String enumPrint;
  private final String enumName;
  private DwTag (int value, String string, String print, String name)
  {
    enumString = string;
    enumValue = value;
    enumPrint = print;
    enumName = name;
  }
  /** Return the qualified name of the enum.  */
  public String toString ()
  {
    return enumString;
  }
  /** Return a printable version of the enum.  */
  public String toPrint ()
  {
    return enumPrint;
  }
  /** Return the name of just the enum.  */
  public String toName ()
  {
    return enumName;
  }
  /**
   * Return true if OBJECT has the same value.
   */
  public boolean equals (Object o)
  {
    if (o == null)
      return false;
    if (! (o instanceof DwTag))
      return false;
    return ((DwTag)o).intValue() == this.intValue();
  }
  public int hashCode ()
  {
    return enumValue;
  }
  public int compareTo (Object o)
  {
    DwTag rhs = (DwTag) o; // Can throw - ok.
    return this.enumValue - rhs.enumValue;
  }
  static public final int ARRAY_TYPE_ = 0x01;
  static public final DwTag ARRAY_TYPE = new DwTag (0x01, "DwTag_ARRAY_TYPE", "DW_TAG_array_type", "ARRAY_TYPE");
  static public final int CLASS_TYPE_ = 0x02;
  static public final DwTag CLASS_TYPE = new DwTag (0x02, "DwTag_CLASS_TYPE", "DW_TAG_class_type", "CLASS_TYPE");
  static public final int ENTRY_POINT_ = 0x03;
  static public final DwTag ENTRY_POINT = new DwTag (0x03, "DwTag_ENTRY_POINT", "DW_TAG_entry_point", "ENTRY_POINT");
  static public final int ENUMERATION_TYPE_ = 0x04;
  static public final DwTag ENUMERATION_TYPE = new DwTag (0x04, "DwTag_ENUMERATION_TYPE", "DW_TAG_enumeration_type", "ENUMERATION_TYPE");
  static public final int FORMAL_PARAMETER_ = 0x05;
  static public final DwTag FORMAL_PARAMETER = new DwTag (0x05, "DwTag_FORMAL_PARAMETER", "DW_TAG_formal_parameter", "FORMAL_PARAMETER");
  static public final int IMPORTED_DECLARATION_ = 0x08;
  static public final DwTag IMPORTED_DECLARATION = new DwTag (0x08, "DwTag_IMPORTED_DECLARATION", "DW_TAG_imported_declaration", "IMPORTED_DECLARATION");
  static public final int LABEL_ = 0x0a;
  static public final DwTag LABEL = new DwTag (0x0a, "DwTag_LABEL", "DW_TAG_label", "LABEL");
  static public final int LEXICAL_BLOCK_ = 0x0b;
  static public final DwTag LEXICAL_BLOCK = new DwTag (0x0b, "DwTag_LEXICAL_BLOCK", "DW_TAG_lexical_block", "LEXICAL_BLOCK");
  static public final int MEMBER_ = 0x0d;
  static public final DwTag MEMBER = new DwTag (0x0d, "DwTag_MEMBER", "DW_TAG_member", "MEMBER");
  static public final int POINTER_TYPE_ = 0x0f;
  static public final DwTag POINTER_TYPE = new DwTag (0x0f, "DwTag_POINTER_TYPE", "DW_TAG_pointer_type", "POINTER_TYPE");
  static public final int REFERENCE_TYPE_ = 0x10;
  static public final DwTag REFERENCE_TYPE = new DwTag (0x10, "DwTag_REFERENCE_TYPE", "DW_TAG_reference_type", "REFERENCE_TYPE");
  static public final int COMPILE_UNIT_ = 0x11;
  static public final DwTag COMPILE_UNIT = new DwTag (0x11, "DwTag_COMPILE_UNIT", "DW_TAG_compile_unit", "COMPILE_UNIT");
  static public final int STRING_TYPE_ = 0x12;
  static public final DwTag STRING_TYPE = new DwTag (0x12, "DwTag_STRING_TYPE", "DW_TAG_string_type", "STRING_TYPE");
  static public final int STRUCTURE_TYPE_ = 0x13;
  static public final DwTag STRUCTURE_TYPE = new DwTag (0x13, "DwTag_STRUCTURE_TYPE", "DW_TAG_structure_type", "STRUCTURE_TYPE");
  static public final int SUBROUTINE_TYPE_ = 0x15;
  static public final DwTag SUBROUTINE_TYPE = new DwTag (0x15, "DwTag_SUBROUTINE_TYPE", "DW_TAG_subroutine_type", "SUBROUTINE_TYPE");
  static public final int TYPEDEF_ = 0x16;
  static public final DwTag TYPEDEF = new DwTag (0x16, "DwTag_TYPEDEF", "DW_TAG_typedef", "TYPEDEF");
  static public final int UNION_TYPE_ = 0x17;
  static public final DwTag UNION_TYPE = new DwTag (0x17, "DwTag_UNION_TYPE", "DW_TAG_union_type", "UNION_TYPE");
  static public final int UNSPECIFIED_PARAMETERS_ = 0x18;
  static public final DwTag UNSPECIFIED_PARAMETERS = new DwTag (0x18, "DwTag_UNSPECIFIED_PARAMETERS", "DW_TAG_unspecified_parameters", "UNSPECIFIED_PARAMETERS");
  static public final int VARIANT_ = 0x19;
  static public final DwTag VARIANT = new DwTag (0x19, "DwTag_VARIANT", "DW_TAG_variant", "VARIANT");
  static public final int COMMON_BLOCK_ = 0x1a;
  static public final DwTag COMMON_BLOCK = new DwTag (0x1a, "DwTag_COMMON_BLOCK", "DW_TAG_common_block", "COMMON_BLOCK");
  static public final int COMMON_INCLUSION_ = 0x1b;
  static public final DwTag COMMON_INCLUSION = new DwTag (0x1b, "DwTag_COMMON_INCLUSION", "DW_TAG_common_inclusion", "COMMON_INCLUSION");
  static public final int INHERITANCE_ = 0x1c;
  static public final DwTag INHERITANCE = new DwTag (0x1c, "DwTag_INHERITANCE", "DW_TAG_inheritance", "INHERITANCE");
  static public final int INLINED_SUBROUTINE_ = 0x1d;
  static public final DwTag INLINED_SUBROUTINE = new DwTag (0x1d, "DwTag_INLINED_SUBROUTINE", "DW_TAG_inlined_subroutine", "INLINED_SUBROUTINE");
  static public final int MODULE_ = 0x1e;
  static public final DwTag MODULE = new DwTag (0x1e, "DwTag_MODULE", "DW_TAG_module", "MODULE");
  static public final int PTR_TO_MEMBER_TYPE_ = 0x1f;
  static public final DwTag PTR_TO_MEMBER_TYPE = new DwTag (0x1f, "DwTag_PTR_TO_MEMBER_TYPE", "DW_TAG_ptr_to_member_type", "PTR_TO_MEMBER_TYPE");
  static public final int SET_TYPE_ = 0x20;
  static public final DwTag SET_TYPE = new DwTag (0x20, "DwTag_SET_TYPE", "DW_TAG_set_type", "SET_TYPE");
  static public final int SUBRANGE_TYPE_ = 0x21;
  static public final DwTag SUBRANGE_TYPE = new DwTag (0x21, "DwTag_SUBRANGE_TYPE", "DW_TAG_subrange_type", "SUBRANGE_TYPE");
  static public final int WITH_STMT_ = 0x22;
  static public final DwTag WITH_STMT = new DwTag (0x22, "DwTag_WITH_STMT", "DW_TAG_with_stmt", "WITH_STMT");
  static public final int ACCESS_DECLARATION_ = 0x23;
  static public final DwTag ACCESS_DECLARATION = new DwTag (0x23, "DwTag_ACCESS_DECLARATION", "DW_TAG_access_declaration", "ACCESS_DECLARATION");
  static public final int BASE_TYPE_ = 0x24;
  static public final DwTag BASE_TYPE = new DwTag (0x24, "DwTag_BASE_TYPE", "DW_TAG_base_type", "BASE_TYPE");
  static public final int CATCH_BLOCK_ = 0x25;
  static public final DwTag CATCH_BLOCK = new DwTag (0x25, "DwTag_CATCH_BLOCK", "DW_TAG_catch_block", "CATCH_BLOCK");
  static public final int CONST_TYPE_ = 0x26;
  static public final DwTag CONST_TYPE = new DwTag (0x26, "DwTag_CONST_TYPE", "DW_TAG_const_type", "CONST_TYPE");
  static public final int CONSTANT_ = 0x27;
  static public final DwTag CONSTANT = new DwTag (0x27, "DwTag_CONSTANT", "DW_TAG_constant", "CONSTANT");
  static public final int ENUMERATOR_ = 0x28;
  static public final DwTag ENUMERATOR = new DwTag (0x28, "DwTag_ENUMERATOR", "DW_TAG_enumerator", "ENUMERATOR");
  static public final int FILE_TYPE_ = 0x29;
  static public final DwTag FILE_TYPE = new DwTag (0x29, "DwTag_FILE_TYPE", "DW_TAG_file_type", "FILE_TYPE");
  static public final int FRIEND_ = 0x2a;
  static public final DwTag FRIEND = new DwTag (0x2a, "DwTag_FRIEND", "DW_TAG_friend", "FRIEND");
  static public final int NAMELIST_ = 0x2b;
  static public final DwTag NAMELIST = new DwTag (0x2b, "DwTag_NAMELIST", "DW_TAG_namelist", "NAMELIST");
  static public final int NAMELIST_ITEM_ = 0x2c;
  static public final DwTag NAMELIST_ITEM = new DwTag (0x2c, "DwTag_NAMELIST_ITEM", "DW_TAG_namelist_item", "NAMELIST_ITEM");
  static public final int PACKED_TYPE_ = 0x2d;
  static public final DwTag PACKED_TYPE = new DwTag (0x2d, "DwTag_PACKED_TYPE", "DW_TAG_packed_type", "PACKED_TYPE");
  static public final int SUBPROGRAM_ = 0x2e;
  static public final DwTag SUBPROGRAM = new DwTag (0x2e, "DwTag_SUBPROGRAM", "DW_TAG_subprogram", "SUBPROGRAM");
  static public final int TEMPLATE_TYPE_PARAMETER_ = 0x2f;
  static public final DwTag TEMPLATE_TYPE_PARAMETER = new DwTag (0x2f, "DwTag_TEMPLATE_TYPE_PARAMETER", "DW_TAG_template_type_parameter", "TEMPLATE_TYPE_PARAMETER");
  static public final int TEMPLATE_VALUE_PARAMETER_ = 0x30;
  static public final DwTag TEMPLATE_VALUE_PARAMETER = new DwTag (0x30, "DwTag_TEMPLATE_VALUE_PARAMETER", "DW_TAG_template_value_parameter", "TEMPLATE_VALUE_PARAMETER");
  static public final int THROWN_TYPE_ = 0x31;
  static public final DwTag THROWN_TYPE = new DwTag (0x31, "DwTag_THROWN_TYPE", "DW_TAG_thrown_type", "THROWN_TYPE");
  static public final int TRY_BLOCK_ = 0x32;
  static public final DwTag TRY_BLOCK = new DwTag (0x32, "DwTag_TRY_BLOCK", "DW_TAG_try_block", "TRY_BLOCK");
  static public final int VARIANT_PART_ = 0x33;
  static public final DwTag VARIANT_PART = new DwTag (0x33, "DwTag_VARIANT_PART", "DW_TAG_variant_part", "VARIANT_PART");
  static public final int VARIABLE_ = 0x34;
  static public final DwTag VARIABLE = new DwTag (0x34, "DwTag_VARIABLE", "DW_TAG_variable", "VARIABLE");
  static public final int VOLATILE_TYPE_ = 0x35;
  static public final DwTag VOLATILE_TYPE = new DwTag (0x35, "DwTag_VOLATILE_TYPE", "DW_TAG_volatile_type", "VOLATILE_TYPE");
  static public final int DWARF_PROCEDURE_ = 0x36;
  static public final DwTag DWARF_PROCEDURE = new DwTag (0x36, "DwTag_DWARF_PROCEDURE", "DW_TAG_dwarf_procedure", "DWARF_PROCEDURE");
  static public final int RESTRICT_TYPE_ = 0x37;
  static public final DwTag RESTRICT_TYPE = new DwTag (0x37, "DwTag_RESTRICT_TYPE", "DW_TAG_restrict_type", "RESTRICT_TYPE");
  static public final int INTERFACE_TYPE_ = 0x38;
  static public final DwTag INTERFACE_TYPE = new DwTag (0x38, "DwTag_INTERFACE_TYPE", "DW_TAG_interface_type", "INTERFACE_TYPE");
  static public final int NAMESPACE_ = 0x39;
  static public final DwTag NAMESPACE = new DwTag (0x39, "DwTag_NAMESPACE", "DW_TAG_namespace", "NAMESPACE");
  static public final int IMPORTED_MODULE_ = 0x3a;
  static public final DwTag IMPORTED_MODULE = new DwTag (0x3a, "DwTag_IMPORTED_MODULE", "DW_TAG_imported_module", "IMPORTED_MODULE");
  static public final int UNSPECIFIED_TYPE_ = 0x3b;
  static public final DwTag UNSPECIFIED_TYPE = new DwTag (0x3b, "DwTag_UNSPECIFIED_TYPE", "DW_TAG_unspecified_type", "UNSPECIFIED_TYPE");
  static public final int PARTIAL_UNIT_ = 0x3c;
  static public final DwTag PARTIAL_UNIT = new DwTag (0x3c, "DwTag_PARTIAL_UNIT", "DW_TAG_partial_unit", "PARTIAL_UNIT");
  static public final int IMPORTED_UNIT_ = 0x3d;
  static public final DwTag IMPORTED_UNIT = new DwTag (0x3d, "DwTag_IMPORTED_UNIT", "DW_TAG_imported_unit", "IMPORTED_UNIT");
  static public final int MUTABLE_TYPE_ = 0x3e;
  static public final DwTag MUTABLE_TYPE = new DwTag (0x3e, "DwTag_MUTABLE_TYPE", "DW_TAG_mutable_type", "MUTABLE_TYPE");
  static public final int CONDITION_ = 0x3f;
  static public final DwTag CONDITION = new DwTag (0x3f, "DwTag_CONDITION", "DW_TAG_condition", "CONDITION");
  static public final int SHARED_TYPE_ = 0x40;
  static public final DwTag SHARED_TYPE = new DwTag (0x40, "DwTag_SHARED_TYPE", "DW_TAG_shared_type", "SHARED_TYPE");
  static public final int LO_USER_ = 0x4080;
  static public final DwTag LO_USER = new DwTag (0x4080, "DwTag_LO_USER", "DW_TAG_lo_user", "LO_USER");
  static public final int MIPS_LOOP_ = 0x4081;
  static public final DwTag MIPS_LOOP = new DwTag (0x4081, "DwTag_MIPS_LOOP", "DW_TAG_MIPS_loop", "MIPS_LOOP");
  static public final int FORMAT_LABEL_ = 0x4101;
  static public final DwTag FORMAT_LABEL = new DwTag (0x4101, "DwTag_FORMAT_LABEL", "DW_TAG_format_label", "FORMAT_LABEL");
  static public final int FUNCTION_TEMPLATE_ = 0x4102;
  static public final DwTag FUNCTION_TEMPLATE = new DwTag (0x4102, "DwTag_FUNCTION_TEMPLATE", "DW_TAG_function_template", "FUNCTION_TEMPLATE");
  static public final int CLASS_TEMPLATE_ = 0x4103;
  static public final DwTag CLASS_TEMPLATE = new DwTag (0x4103, "DwTag_CLASS_TEMPLATE", "DW_TAG_class_template", "CLASS_TEMPLATE");
  static public final int HI_USER_ = 0xffff;
  static public final DwTag HI_USER = new DwTag (0xffff, "DwTag_HI_USER", "DW_TAG_hi_user", "HI_USER");

  /** Create a HashMap containing all the DwTag elements.  */
  private static java.util.Map getMap ()
  {
    java.util.Map map;
    map = new java.util.HashMap ();
    map.put (ARRAY_TYPE.enumName, ARRAY_TYPE);
    map.put (CLASS_TYPE.enumName, CLASS_TYPE);
    map.put (ENTRY_POINT.enumName, ENTRY_POINT);
    map.put (ENUMERATION_TYPE.enumName, ENUMERATION_TYPE);
    map.put (FORMAL_PARAMETER.enumName, FORMAL_PARAMETER);
    map.put (IMPORTED_DECLARATION.enumName, IMPORTED_DECLARATION);
    map.put (LABEL.enumName, LABEL);
    map.put (LEXICAL_BLOCK.enumName, LEXICAL_BLOCK);
    map.put (MEMBER.enumName, MEMBER);
    map.put (POINTER_TYPE.enumName, POINTER_TYPE);
    map.put (REFERENCE_TYPE.enumName, REFERENCE_TYPE);
    map.put (COMPILE_UNIT.enumName, COMPILE_UNIT);
    map.put (STRING_TYPE.enumName, STRING_TYPE);
    map.put (STRUCTURE_TYPE.enumName, STRUCTURE_TYPE);
    map.put (SUBROUTINE_TYPE.enumName, SUBROUTINE_TYPE);
    map.put (TYPEDEF.enumName, TYPEDEF);
    map.put (UNION_TYPE.enumName, UNION_TYPE);
    map.put (UNSPECIFIED_PARAMETERS.enumName, UNSPECIFIED_PARAMETERS);
    map.put (VARIANT.enumName, VARIANT);
    map.put (COMMON_BLOCK.enumName, COMMON_BLOCK);
    map.put (COMMON_INCLUSION.enumName, COMMON_INCLUSION);
    map.put (INHERITANCE.enumName, INHERITANCE);
    map.put (INLINED_SUBROUTINE.enumName, INLINED_SUBROUTINE);
    map.put (MODULE.enumName, MODULE);
    map.put (PTR_TO_MEMBER_TYPE.enumName, PTR_TO_MEMBER_TYPE);
    map.put (SET_TYPE.enumName, SET_TYPE);
    map.put (SUBRANGE_TYPE.enumName, SUBRANGE_TYPE);
    map.put (WITH_STMT.enumName, WITH_STMT);
    map.put (ACCESS_DECLARATION.enumName, ACCESS_DECLARATION);
    map.put (BASE_TYPE.enumName, BASE_TYPE);
    map.put (CATCH_BLOCK.enumName, CATCH_BLOCK);
    map.put (CONST_TYPE.enumName, CONST_TYPE);
    map.put (CONSTANT.enumName, CONSTANT);
    map.put (ENUMERATOR.enumName, ENUMERATOR);
    map.put (FILE_TYPE.enumName, FILE_TYPE);
    map.put (FRIEND.enumName, FRIEND);
    map.put (NAMELIST.enumName, NAMELIST);
    map.put (NAMELIST_ITEM.enumName, NAMELIST_ITEM);
    map.put (PACKED_TYPE.enumName, PACKED_TYPE);
    map.put (SUBPROGRAM.enumName, SUBPROGRAM);
    map.put (TEMPLATE_TYPE_PARAMETER.enumName, TEMPLATE_TYPE_PARAMETER);
    map.put (TEMPLATE_VALUE_PARAMETER.enumName, TEMPLATE_VALUE_PARAMETER);
    map.put (THROWN_TYPE.enumName, THROWN_TYPE);
    map.put (TRY_BLOCK.enumName, TRY_BLOCK);
    map.put (VARIANT_PART.enumName, VARIANT_PART);
    map.put (VARIABLE.enumName, VARIABLE);
    map.put (VOLATILE_TYPE.enumName, VOLATILE_TYPE);
    map.put (DWARF_PROCEDURE.enumName, DWARF_PROCEDURE);
    map.put (RESTRICT_TYPE.enumName, RESTRICT_TYPE);
    map.put (INTERFACE_TYPE.enumName, INTERFACE_TYPE);
    map.put (NAMESPACE.enumName, NAMESPACE);
    map.put (IMPORTED_MODULE.enumName, IMPORTED_MODULE);
    map.put (UNSPECIFIED_TYPE.enumName, UNSPECIFIED_TYPE);
    map.put (PARTIAL_UNIT.enumName, PARTIAL_UNIT);
    map.put (IMPORTED_UNIT.enumName, IMPORTED_UNIT);
    map.put (MUTABLE_TYPE.enumName, MUTABLE_TYPE);
    map.put (CONDITION.enumName, CONDITION);
    map.put (SHARED_TYPE.enumName, SHARED_TYPE);
    map.put (LO_USER.enumName, LO_USER);
    map.put (MIPS_LOOP.enumName, MIPS_LOOP);
    map.put (FORMAT_LABEL.enumName, FORMAT_LABEL);
    map.put (FUNCTION_TEMPLATE.enumName, FUNCTION_TEMPLATE);
    map.put (CLASS_TEMPLATE.enumName, CLASS_TEMPLATE);
    map.put (HI_USER.enumName, HI_USER);
    return map;
  }
  private static java.util.Map enumMap = getMap ();

  /** Return the DwTag object that matches the string.  */
  public static DwTag valueOf (String string)
  {
    return (DwTag)enumMap.get (string);
  }

  /** Return the DwTag object that matches the integer.  */
  public static DwTag valueOf (long i)
  {
    switch ((int)i) {
    case ARRAY_TYPE_: return ARRAY_TYPE;
    case CLASS_TYPE_: return CLASS_TYPE;
    case ENTRY_POINT_: return ENTRY_POINT;
    case ENUMERATION_TYPE_: return ENUMERATION_TYPE;
    case FORMAL_PARAMETER_: return FORMAL_PARAMETER;
    case IMPORTED_DECLARATION_: return IMPORTED_DECLARATION;
    case LABEL_: return LABEL;
    case LEXICAL_BLOCK_: return LEXICAL_BLOCK;
    case MEMBER_: return MEMBER;
    case POINTER_TYPE_: return POINTER_TYPE;
    case REFERENCE_TYPE_: return REFERENCE_TYPE;
    case COMPILE_UNIT_: return COMPILE_UNIT;
    case STRING_TYPE_: return STRING_TYPE;
    case STRUCTURE_TYPE_: return STRUCTURE_TYPE;
    case SUBROUTINE_TYPE_: return SUBROUTINE_TYPE;
    case TYPEDEF_: return TYPEDEF;
    case UNION_TYPE_: return UNION_TYPE;
    case UNSPECIFIED_PARAMETERS_: return UNSPECIFIED_PARAMETERS;
    case VARIANT_: return VARIANT;
    case COMMON_BLOCK_: return COMMON_BLOCK;
    case COMMON_INCLUSION_: return COMMON_INCLUSION;
    case INHERITANCE_: return INHERITANCE;
    case INLINED_SUBROUTINE_: return INLINED_SUBROUTINE;
    case MODULE_: return MODULE;
    case PTR_TO_MEMBER_TYPE_: return PTR_TO_MEMBER_TYPE;
    case SET_TYPE_: return SET_TYPE;
    case SUBRANGE_TYPE_: return SUBRANGE_TYPE;
    case WITH_STMT_: return WITH_STMT;
    case ACCESS_DECLARATION_: return ACCESS_DECLARATION;
    case BASE_TYPE_: return BASE_TYPE;
    case CATCH_BLOCK_: return CATCH_BLOCK;
    case CONST_TYPE_: return CONST_TYPE;
    case CONSTANT_: return CONSTANT;
    case ENUMERATOR_: return ENUMERATOR;
    case FILE_TYPE_: return FILE_TYPE;
    case FRIEND_: return FRIEND;
    case NAMELIST_: return NAMELIST;
    case NAMELIST_ITEM_: return NAMELIST_ITEM;
    case PACKED_TYPE_: return PACKED_TYPE;
    case SUBPROGRAM_: return SUBPROGRAM;
    case TEMPLATE_TYPE_PARAMETER_: return TEMPLATE_TYPE_PARAMETER;
    case TEMPLATE_VALUE_PARAMETER_: return TEMPLATE_VALUE_PARAMETER;
    case THROWN_TYPE_: return THROWN_TYPE;
    case TRY_BLOCK_: return TRY_BLOCK;
    case VARIANT_PART_: return VARIANT_PART;
    case VARIABLE_: return VARIABLE;
    case VOLATILE_TYPE_: return VOLATILE_TYPE;
    case DWARF_PROCEDURE_: return DWARF_PROCEDURE;
    case RESTRICT_TYPE_: return RESTRICT_TYPE;
    case INTERFACE_TYPE_: return INTERFACE_TYPE;
    case NAMESPACE_: return NAMESPACE;
    case IMPORTED_MODULE_: return IMPORTED_MODULE;
    case UNSPECIFIED_TYPE_: return UNSPECIFIED_TYPE;
    case PARTIAL_UNIT_: return PARTIAL_UNIT;
    case IMPORTED_UNIT_: return IMPORTED_UNIT;
    case MUTABLE_TYPE_: return MUTABLE_TYPE;
    case CONDITION_: return CONDITION;
    case SHARED_TYPE_: return SHARED_TYPE;
    case LO_USER_: return LO_USER;
    case MIPS_LOOP_: return MIPS_LOOP;
    case FORMAT_LABEL_: return FORMAT_LABEL;
    case FUNCTION_TEMPLATE_: return FUNCTION_TEMPLATE;
    case CLASS_TEMPLATE_: return CLASS_TEMPLATE;
    case HI_USER_: return HI_USER;
    default: return null;
    }
  }

  /** Return an array of all the DwTag elements.  */
  public static DwTag[] values ()
  {
    return (DwTag[]) enumMap.values ().toArray (new DwTag[0]);
  }

  /**
   * Returns the full underscore delimited name of the
   * field corresponding to the value I.
   */
  static public String toString (long i)
  {
    DwTag c = valueOf (i);
    if (c == null)
      return "_0x" + Long.toHexString (i);
    else
      return c.toString ();
  }

  /**
   * Returns the printable (or user readable) name for the
   * field corresponding to the value I.
   */
  static public String toPrintString (long i)
  {
    DwTag c = valueOf (i);
    if (c == null)
      return "_0x" + Long.toHexString (i);
    else
      return c.toPrint ();
  }

  /**
   * Returns the printable (or user readable) name for the
   * field corresponding to the value I, or DEF is there
   * is no such field.
   */
  static public String toPrintString (long i, String def)
  {
    DwTag c = valueOf (i);
    if (c == null)
      return def;
    else
      return c.toPrint ();
  }

  /**
   * Returns just the name part of the num corresponding to I.
   */
  static public String toName (long i)
  {
    DwTag c = valueOf (i);
    if (c == null)
      return "_0x" + Long.toHexString (i);
    else
      return c.toName ();
  }

  /**
   * Returns just the name part of the num corresponding to I,
   * or DEF is there is no such field.
   */
  static public String toName (long i, String def)
  {
    DwTag c = valueOf (i);
    if (c == null)
      return def;
    else
      return c.toName ();
  }

  /**
   * Return the equivalent of the enum.
   */
  public int intValue () {
    return enumValue;
  }

  /**
   * Return the equivalent of the enum.
   */
  public long longValue () {
    return enumValue;
  }

  /**
   * Return the equivalent of the enum.
   */
  public float floatValue () {
    return enumValue;
  }

  /**
   * Return the equivalent of the enum.
   */
  public double doubleValue () {
    return enumValue;
  }

  static final long serialVersionUID = 0;
}
